local scripts = {
	checkbutton = {
		OnShow = function() this:SetChecked(this.self:Get(this.setting)) end,
		OnClick = function() this.self:Set(this.setting,this:GetChecked() and true or false); this.self:UpdateSettings() end,
	},
	slider = {
		OnShow = function() this.real = false this:SetValue(this.self:Get(this.setting)) this.real = true end,
		OnValueChanged = function()
			local value = this:GetValue()
			local formatstring
			if math.floor(value) == value then 
				formatstring = "%d"
			else
				formatstring = "%.2f"
			end
			getglobal(this:GetName().."Value"):SetText(string.format(formatstring,value))
			if this.real then
				this.self:Set(this.setting,value)
				this.self:UpdateSettings()
			end
		end,
	},
	editbox1 = { --initialized with a string
		OnEditFocusGained = function() this:SetText(this.self:Get(this.setting) or ""); this:HighlightText() end,
		OnEditFocusLost = function() this:HighlightText(0,0) end,
		OnShow = function() this:SetText(this.self:Get(this.setting) or "") end,
		OnEscapePressed = function()
			local text = this:GetText()
			if text == "" then
				this:ClearFocus()
			else
				this:SetText("")
			end
		end,
		OnEnterPressed = function() 
			local text = this:GetText()
			this.self:Print("|cff00ffff",this.self:GetName(),"|r: "..ASH_New_setting,text)
			this.self:Set(this.setting,text); this.self:UpdateSettings(); this:ClearFocus() 
		end,
	},
	editbox2 = { --left blank, OnEnterPressed and OnEnter will be defined elsewhere
		OnEditFocusGained = function() this:SetText("") end,
		OnEditFocusLost = function() this:SetText("") end,
		OnEscapePressed = function()
			local text = this:GetText()
			if text == "" then
				this:ClearFocus()
			else
				this:SetText("")
			end
		end,
	},
	dropdown = {
		OnShow = function()
			getglobal(this:GetName().."Text"):SetText(this.text)
		end,
	},
}

local sortingtable = {}
local function sorttable(pretable)
	for index,value in pairs(sortingtable) do sortingtable[index] = nil end
	for index,value in pairs(pretable) do table.insert(sortingtable,index) end
	table.sort(sortingtable)
	return sortingtable
end

local types = {
	[ASH_Harmful] = "debuff",
	[ASH_Helpful] = "buff",
	[ASH_NoTarget] = "notarget",
}

local orderedtypes = {
	ASH_Harmful,
	ASH_Helpful,
	ASH_NoTarget,
}

local barmsgcodes = {
	["%s"] = ASH_barmsgcodes_s_DOT,
	["%r"] = ASH_barmsgcodes_r_DOT,
	["%d"] = ASH_barmsgcodes_d_DOT,
	["%c"] = ASH_barmsgcodes_c_DOT,
	["%t"] = ASH_barmsgcodes_t_DOT,
	["%l"] = ASH_barmsgcodes_l_DOT,
}

local category

local function colordropdown()
	local info
	local DROPDOWNLIB_MENU_LEVEL = DropDownLib:MenuLevel()
	local DROPDOWNLIB_MENU_VALUE = DropDownLib:MenuValue()
	if DROPDOWNLIB_MENU_LEVEL == 4 then
		local subcat = DROPDOWNLIB_MENU_VALUE
		for index,value in ipairs({ASH_begin,ASH_half,ASH_final}) do
			info = DropDownLib:GetTable()
			info.text = value
			info.notCheckable = 1
			info.hasColorSwatch = 1
			info.r = DoTimer:Get("colors",category,subcat,value,"r") or 1
			info.g = DoTimer:Get("colors",category,subcat,value,"g") or 1
			info.b = DoTimer:Get("colors",category,subcat,value,"b") or 1
			info.swatchFunc = function()
				local r,g,b = DropDownLibColorPicker:GetColorRGB()
				DoTimer:Set("colors",category,subcat,value,"r",r)
				DoTimer:Set("colors",category,subcat,value,"g",g)
				DoTimer:Set("colors",category,subcat,value,"b",b)
				for i,v in ipairs({ASH_begin,ASH_half,ASH_final}) do
					if not DoTimer:Get("colors",category,subcat,v) then
						DoTimer:Set("colors",category,subcat,v,"r",r)
						DoTimer:Set("colors",category,subcat,v,"g",g)
						DoTimer:Set("colors",category,subcat,v,"b",b)
					end
				end
				DoTimer:UpdateSettings()
			end
			DropDownLib:AddButton(info,4)
		end
	elseif DROPDOWNLIB_MENU_LEVEL == 3 then
		if category == "timer" then
			local entries = DoTimer:AcquireTable()
			local tab = DROPDOWNLIB_MENU_VALUE
			for i = 1,GetNumSpellTabs() do
				local name,_,offset,numspells = GetSpellTabInfo(i)
				if name == tab then
					for id = offset + 1,offset + numspells do
						local spell = GetSpellName(id,BOOKTYPE_SPELL)
						entries[spell] = 1
					end
					break
				end
			end
			local seductionname = DoTimer:ToLocale("Seduction")
			if not (seductionname == "unknown") then entries[seductionname] = 1 end
			entries = sorttable(entries)
			for index,value in ipairs(entries) do
				info = DropDownLib:GetTable()
				info.text = value
				local checked
				if DoTimer:Get("colors","timer",value) then checked = 1 end
				info.checked = checked
				info.value = value
				info.hasArrow = 1
				info.func = function() 
					if checked then 
						DoTimer:Set("colors","timer",value,nil) 
						DoTimer:UpdateSettings()
					end 
				end
				DropDownLib:AddButton(info,3)
			end
		elseif category == "type" then
			local subcat = DROPDOWNLIB_MENU_VALUE
			for index,value in ipairs({ASH_begin,ASH_half,ASH_final}) do
				info = DropDownLib:GetTable()
				info.text = value
				info.notCheckable = 1
				info.hasColorSwatch = 1
				info.r = DoTimer:Get("colors",category,subcat,value,"r") or DoTimer:GetTimerData()[subcat].colors[value]["r"]
				info.g = DoTimer:Get("colors",category,subcat,value,"g") or DoTimer:GetTimerData()[subcat].colors[value]["g"]
				info.b = DoTimer:Get("colors",category,subcat,value,"b") or DoTimer:GetTimerData()[subcat].colors[value]["b"]
				info.swatchFunc = function()
					local r,g,b = DropDownLibColorPicker:GetColorRGB()
					DoTimer:Set("colors",category,subcat,value,"r",r)
					DoTimer:Set("colors",category,subcat,value,"g",g)
					DoTimer:Set("colors",category,subcat,value,"b",b)
					for i,v in ipairs({ASH_begin,ASH_half,ASH_final}) do
						if not DoTimer:Get("colors",category,subcat,v) then
							DoTimer:Set("colors",category,subcat,v,"r",DoTimer:GetTimerData()[subcat].colors[v]["r"])
							DoTimer:Set("colors",category,subcat,v,"g",DoTimer:GetTimerData()[subcat].colors[v]["g"])
							DoTimer:Set("colors",category,subcat,v,"b",DoTimer:GetTimerData()[subcat].colors[v]["b"])
						end
					end
					DoTimer:UpdateSettings()
				end
				DropDownLib:AddButton(info,3)
			end
		end
	elseif DROPDOWNLIB_MENU_LEVEL == 2 then
		category = DROPDOWNLIB_MENU_VALUE
		if category == "timer" then
			for i = 1,GetNumSpellTabs() do
				local name = GetSpellTabInfo(i)
				info = DropDownLib:GetTable()
				info.text = name
				info.value = name
				info.hasArrow = 1
				info.notCheckable = 1
				DropDownLib:AddButton(info,2)
			end
		elseif category == "type" then
			for index,value in ipairs(orderedtypes) do
				info = DropDownLib:GetTable()
				info.text = value
				local timertype = types[value]
				local checked
				if DoTimer:Get("colors","type",timertype) then checked = 1 end
				info.checked = checked
				info.value = timertype
				info.hasArrow = 1
				info.func = function() 
					if checked then 
						DoTimer:Set("colors","type",timertype,nil) 
						DoTimer:UpdateSettings()
					end 
				end
				DropDownLib:AddButton(info,2)
			end
		end
	elseif DROPDOWNLIB_MENU_LEVEL == 1 then
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Specific_Timer
		info.notCheckable = 1
		info.hasArrow = 1
		info.value = "timer"
		DropDownLib:AddButton(info,1)
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Timer_Type
		info.notCheckable = 1
		info.hasArrow = 1
		info.value = "type"
		DropDownLib:AddButton(info,1)
	end
end

local guitable = {
	title = ASH_Main_Menu_Dot,
	name = "DoTimerMenuFrame",
	{ --column 1
		{
			title = ASH_Enabled,
			type = "checkbutton",
			tooltiptext = ASH_Enabled_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "status",
			},
		},
		{
			title = ASH_levels,
			type = "checkbutton",
			tooltiptext = ASH_levels_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "levels",
			},
		},
		{
			title = ASH_Tooltips,
			type = "checkbutton",
			tooltiptext = ASH_Tooltips_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "tooltips",
			},
		},
		{
			title = ASH_On_Yourself,
			type = "checkbutton",
			tooltiptext = ASH_On_Yourself_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "playertimers",
			},
		},
		{
			title = ASH_Show_Only_Target,
			type = "checkbutton",
			tooltiptext = ASH_Show_Only_Target_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "onlytarget",
			},
		},
		{
			title = ASH_Sort_Target,
			type = "checkbutton",
			tooltiptext = ASH_Sort_Target_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "sorttarget",
			},
		},
		{
			title = ASH_SHOW_Only_Focus,
			type = "checkbutton",
			tooltiptext = ASH_SHOW_Only_Focus_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "onlyfocus",
			},
		},
		{
			title = ASH_Sort_Focus,
			type = "checkbutton",
			tooltiptext = ASH_Sort_Focus_tooltiptext,
			scripts = scripts.checkbutton,
			data = {
				setting = "sortfocus",
			},
		},
	},
	{ --column 2
		{
			title = ASH_Max_Buff_Duration,
			type = "slider",
			tooltiptext = ASH_Max_Buff_Duration_tooltiptext,
			scripts = scripts.slider,
			lowval = 0,
			highval = 15,
			step = 1,
			data = {
				setting = "maxbuffduration",
			},
		},
		{
			title = ASH_Colors,
			type = "dropdown",
			tooltiptext = ASH_Colors_tooltiptext,
			func = colordropdown,
			scripts = scripts.dropdown,
			data = {
				text = ASH_Choose_text,
			},
		},
		{
			title = ASH_Key_Bindings,
			type = "dropdown",
			tooltiptext = ASH_Key_Bindings_tooltiptext,
			func = function() DoTimer:KBDropDown() end,
			scripts = scripts.dropdown,
			data = {
				text = ASH_Choose_text,
			},
		},
		{
		title = ASH_Debugging,
			type = "editbox",
			tooltiptext = ASH_Debugging_tooltiptext,
			scripts = scripts.editbox1,
			data = {
				setting = "debugchannel",
			},
		},
		{
			title = ASH_Chat_Msg,
			type = "editbox",
			tooltiptext = ASH_Chat_Msg_tooltiptext,
			scripts = scripts.editbox2,
			overridescripts = {
				OnEnterPressed = function() 
					local text = this:GetText()
					if not (text == "") then 
						local _,_,output,msg = string.find(text,"(%a+)%:%s?(.+)")
						if output then 
							output = string.lower(output)
							DoTimer:Set("chatmsg"..output,msg)
							DoTimer:UpdateSettings()
							this:SetText("") 
							DoTimer:Print(ASH_chat_msg,msg)
						else
							DoTimer:Print("|cff00ffffDoTimer|r: ",output,ASH_Setting_failed)
						end
					end
				end,
				OnEnter = function() 
					if this.tooltipText then
						local text = this.tooltipText
						text = text.."\n"..ASH_Chat_Msg_Normal..DoTimer:Get("chatmsgnormal")
						text = text.."\n"..ASH_Chat_Msg_NoTarget..DoTimer:Get("chatmsgnotarget")
						GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
						GameTooltip:SetText(text, nil, nil, nil, nil, 1);
					end
				end,
			},
		},
	},
	{ --column 3
		{
			title = ASH_Reset,
			type = "button",
			tooltiptext = ASH_Reset_tooltiptext,
			buffer = 10,
			scripts = {
				OnClick = function() 
					this.self:ClearSettings() 
					DoTimerMenuFrame:Hide()
					DoTimerMenuFrame:Show()
				end,
			},
		},
		{
			title = ASH_Author_Info,
			type = "button",
			tooltiptext = ASH_Author_Info_tooltiptext,
			buffer = 10,
			scripts = {
				OnClick = function() DoTimer:AddAuthorInfo() end,
			},
		},
		{
			title = ASH_Profiles,
			type = "button",
			tooltiptext = ASH_Profiles_tooltiptext,
			buffer = 10,
			scripts = {
				OnClick = function() ProfileLib:ShowGUI() end,
			},
		},
		"BREAK",
		{
			title = ASH_Timer_Settings,
			type = "button",
			tooltiptext = ASH_Timer_Settings_tooltiptext,
			scripts = {
				OnClick = function() this.self:ShowTimerGUI(barmsgcodes,orderedtypes,types) end,
			},
		},
		{
			title = ASH_Close,
			type = "button",
			tooltiptext = ASH_Close_tooltiptext,
			scripts = {
				OnClick = function() this.holder.place.column.menu:Hide() end,
			},
		},
	},
}

function DoTimer:ShowGUI()
	if not DoTimerMenuFrame then
		DoTimer:CreateGUI(guitable)
	end
	DoTimerMenuFrame:Show()
end
		if(wsRegisterButton) then
			wsRegisterButton(
			WS_CONFIG_SEC_1,
			"Combat",
			DOTIMER_TITLENAME,
			"DoTimer",
			"DoTimer",
			"Interface\\Icons\\INV_Misc_Idol_05",
			function()
				DoTimer:ShowGUI(); 
			end);
		end 