ProfileLib = AsheylaLib:NewModule("ProfileLib","GUILib")
local modules = {}
local selectedmodules = {}

function ProfileLib:RegisterForProfiles(module)
	table.insert(modules,module)
end

function ProfileLib:LoadProfile(profile)
	if type(profile) == "string" then
		for i,v in ipairs(self:Get("profiles")) do
			if v.name == profile then
				profile = v
				break
			end
		end
	end
	if profile and type(profile) == "table" then
		for i,v in pairs(profile.modules) do
			local module = AsheylaLib:GetModule(i)
			module:RegisterSettings()
			self:ClearTable(module:GetSettings())
			for index,value in pairs(v) do
				module:Set(index,value)
			end
			module:UpdateSettings()
		end
		self:Print(ASH_Profiles_P1,profile.name,ASH_Profiles_P2)
	end
end

local flagbuttons = {
	["ProfileLibEnterCombatCheckButton"] = "combat-true",
	["ProfileLibLeaveCombatCheckButton"] = "combat-false",
	["ProfileLibJoinGroupCheckButton"] = "group-true",
	["ProfileLibLeaveGroupCheckButton"] = "group-false",
	["ProfileLibUseOtherwiseCheckButton"] = "otherwise",
	["ProfileLibDefaultProfileCheckButton"] = "default",
}

function ProfileLib:InitiateProfileCreation()
	ProfileLibNameEditBox:Show()
	ProfileLibModulesDropDown:Show()
	ProfileLibOnlyCharCheckButton:Show()
	ProfileLibEnterCombatCheckButton:Show()
	ProfileLibLeaveCombatCheckButton:Show()
	ProfileLibJoinGroupCheckButton:Show()
	ProfileLibLeaveGroupCheckButton:Show()
	ProfileLibUseOtherwiseCheckButton:Show()
	ProfileLibDefaultProfileCheckButton:Show()
	ProfileLibDoneButton:Show()
	for i,v in pairs(selectedmodules) do selectedmodules[i] = nil end
end

function ProfileLib:ClearMenu()
	ProfileLibNameEditBox:Hide()
	ProfileLibNameEditBox:SetText("")
	ProfileLibModulesDropDown:Hide()
	ProfileLibOnlyCharCheckButton:Hide()
	ProfileLibEnterCombatCheckButton:Hide()
	ProfileLibLeaveCombatCheckButton:Hide()
	ProfileLibJoinGroupCheckButton:Hide()
	ProfileLibLeaveGroupCheckButton:Hide()
	ProfileLibUseOtherwiseCheckButton:Hide()
	ProfileLibDefaultProfileCheckButton:Hide()
	ProfileLibDoneButton:Hide()
end

function ProfileLib:CreateProfile()
	local newprofile = {
		modules = {},
		flags = {},
	}
	newprofile.name = ProfileLibNameEditBox:GetText()
	if newprofile.name == "" then
		self:Print(ASH_Profiles_Nametext)
		return
	end
	local hasmodules
	for i,v in pairs(selectedmodules) do
		hasmodules = true
		local settings = self:CopyTable(i:GetSettings(),1)
		newprofile.modules[i:GetName()] = settings
	end
	if not hasmodules then
		self:Print(ASH_Profiles_selectmodules)
		return
	end
	if ProfileLibOnlyCharCheckButton:GetChecked() then newprofile.onlychar = GetRealmName().."-"..UnitName("player") end
	for i,v in pairs(flagbuttons) do
		if getglobal(i):GetChecked() then newprofile.flags[v] = 1 end
	end
	self:Set("profiles","+",newprofile)
	self:Print(ASH_Profiles_createdsuccessfully)
	self:ClearMenu()
end

local sortfunc = function(a,b) return a.name < b.name end

local function dropdown()
	local DROPDOWNLIB_MENU_LEVEL = DropDownLib:MenuLevel()
	local DROPDOWNLIB_MENU_VALUE = DropDownLib:MenuValue()
	local info
	if DROPDOWNLIB_MENU_LEVEL == 3 then
		info = DropDownLib:GetTable()
		info.text = ASH_Profiles_LoadMe
		info.notCheckable = 1
		info.textR = .2
		info.textG = 1
		info.textB = .2
		info.arg1 = DROPDOWNLIB_MENU_VALUE
		info.func = function(arg1) ProfileLib:LoadProfile(arg1) DropDownLib:CloseDropDownMenus() end
		DropDownLib:AddButton(info,3)
		info = DropDownLib:GetTable()
		info.text = ""
		info.disabled = 1
		DropDownLib:AddButton(info,3)
		info = DropDownLib:GetTable()
		info.text = ASH_Profiles_Remove_This_Entry
		info.arg1 = DROPDOWNLIB_MENU_VALUE
		info.notCheckable = 1
		info.textR = 1
		info.textG = .3
		info.textB = .3
		info.func = function(arg1)
			for i,v in ipairs(ProfileLib:Get("profiles")) do
				if v == arg1 then
					ProfileLib:Set("profiles","-",i)
					DropDownLib:CloseDropDownMenus()
					return
				end
			end
		end
		DropDownLib:AddButton(info,3)
	elseif DROPDOWNLIB_MENU_LEVEL == 2 then
		local profiles = ProfileLib:Get("profiles")
		if profiles then
			local generalprofs = {}
			local charprofs = {}
			local id = GetRealmName().."-"..UnitName("player")
			for i,v in ipairs(profiles) do
				if (not v.onlychar) then
					table.insert(generalprofs,v)
				elseif (v.onlychar == id) then 
					table.insert(charprofs,v) 
				end
			end
			table.sort(generalprofs,sortfunc)
			table.sort(charprofs,sortfunc)
			if #(charprofs) > 0 then
				info = DropDownLib:GetTable()
				info.text = ASH_Profiles_CharacterSpecific_Profiles
				info.isTitle = 1
				info.notCheckable = 1
				DropDownLib:AddButton(info,2)
				for i,v in ipairs(charprofs) do
					info = DropDownLib:GetTable()
					info.text = v.name
					info.value = v
					info.notCheckable = 1
					info.hasArrow = 1
					DropDownLib:AddButton(info,2)
				end
			end
			if #(generalprofs) > 0 then
				info = DropDownLib:GetTable()
				info.text = ASH_Profiles_General_Profiles
				info.isTitle = 1
				info.notCheckable = 1
				DropDownLib:AddButton(info,2)
				for i,v in ipairs(generalprofs) do
					info = DropDownLib:GetTable()
					info.text = v.name
					info.value = v
					info.notCheckable = 1
					info.hasArrow = 1
					DropDownLib:AddButton(info,2)
				end
			end
			info = DropDownLib:GetTable()
			info.text = ""
			info.disabled = 1
			DropDownLib:AddButton(info,2)
			info = DropDownLib:GetTable()
			info.text = ASH_Profiles_Delete_All_Profiles
			info.notCheckable = 1
			info.textR = 1
			info.textG = .3
			info.textB = .3
			info.func = function()
				ProfileLib:Set("profiles",nil)
				DropDownLib:CloseDropDownMenus()
			end
			DropDownLib:AddButton(info,2)
		end
	elseif DROPDOWNLIB_MENU_LEVEL == 1 then
		info = DropDownLib:GetTable()
		info.notCheckable = 1
		info.text = ASH_Profiles_Create_New
		info.func = function() ProfileLib:InitiateProfileCreation() end
		DropDownLib:AddButton(info,1)
		info = DropDownLib:GetTable()
		info.notCheckable = 1
		info.hasArrow = 1
		info.text = ASH_Profiles_LoadorDelete
		DropDownLib:AddButton(info,1)
	end
end

local scripts = {
	checkbutton = {
		OnShow = function() this:SetChecked(false) end,
	},
}

local guitable = {
	title = ASH_Profiles_Profile_Creator,
	name = "ProfileMenuFrame",
	scripts = {
		OnShow = function() ProfileLib:ClearMenu() end,
	},
	{ --column 1
		{
			title = ASH_Profiles_BeginHere,
			type = "dropdown",
			tooltiptext = ASH_Profiles_BeginHere_text,
			func = function() 
				dropdown()
			end,
			scripts = {
				OnShow = function() getglobal(this:GetName().."Text"):SetText(this.text) end,
			},
			data = {
				text = ASH_Choose_text
			},
		},
		{
			title = ASH_Profiles_Profile_Name,
			name = "ProfileLibNameEditBox",
			type = "editbox",
			tooltiptext = ASH_Profiles_Profile_Name_tooltiptext,
			scripts = {
				OnEditFocusGained = function() this:HighlightText() end,
				OnEditFocusLost = function() this:HighlightText(0,0) end,
				OnEscapePressed = function()
					local text = this:GetText()
					if text == "" then
						this:ClearFocus()
					else
						this:SetText("")
					end
				end,
				OnEnterPressed = function() this:ClearFocus() end,
			},
		},
		{
			title = ASH_Profiles_Modules,
			name = "ProfileLibModulesDropDown",
			type = "dropdown",
			tooltiptext = ASH_Profiles_Modules_tooltiptext,
			func = function()
				local info
				for index,value in ipairs(modules) do
					info = DropDownLib:GetTable()
					info.text = value:GetName()
					info.checked = selectedmodules[value]
					info.arg1 = value
					info.keepShownOnClick = 1
					info.func = function(arg1)
						if this.checked then
							selectedmodules[arg1] = nil
						else
							selectedmodules[arg1] = 1
						end
					end
					DropDownLib:AddButton(info,1)
				end
				info = DropDownLib:GetTable()
				info.text = ""
				info.disabled = 1
				DropDownLib:AddButton(info,1)
				info = DropDownLib:GetTable()
				info.text = ASH_Profiles_Modules_Done
				info.notCheckable = 1
				info.textR = .2
				info.textG = 1
				info.textB = .2
				info.func = function()
					DropDownLib:CloseDropDownMenus()
				end
				DropDownLib:AddButton(info,1)
			end,
			scripts = {
				OnShow = function() getglobal(this:GetName().."Text"):SetText(this.text) end,
			},
			data = {
				text = ASH_Choose_text
			},
		},
		{
			title = ASH_Profiles_OnlyThisCharacter,
			name = "ProfileLibOnlyCharCheckButton",
			type = "checkbutton",
			tooltiptext = ASH_Profiles_OnlyThisCharacter_tooltiptext,
			scripts = scripts.checkbutton,
		},
		{
			title = ASH_Profiles_UseWhenEnteringCombat,
			name = "ProfileLibEnterCombatCheckButton",
			type = "checkbutton",
			tooltiptext = ASH_Profiles_DefaultProfile_tooltiptext,
			scripts = scripts.checkbutton,
			overridescripts = {
				OnClick = function() if this:GetChecked() then ProfileLibLeaveCombatCheckButton:SetChecked(false) end end,
			},
		},
		{
			title = ASH_Profiles_UseWhenLeavingCombat,
			name = "ProfileLibLeaveCombatCheckButton",
			type = "checkbutton",
			tooltiptext = ASH_Profiles_UseWhenLeavingCombat_tooltiptext,
			scripts = scripts.checkbutton,
			overridescripts = {
				OnClick = function() if this:GetChecked() then ProfileLibEnterCombatCheckButton:SetChecked(false) end end,
			},
		},
		{
			title = ASH_Profiles_UseWhenJoiningGroup,
			name = "ProfileLibJoinGroupCheckButton",
			type = "checkbutton",
			scripts = scripts.checkbutton,
			overridescripts = {
				OnClick = function() if this:GetChecked() then ProfileLibLeaveGroupCheckButton:SetChecked(false) end end,
			},
			tooltiptext = ASH_Profiles_UseWhenJoiningGroup_tooltiptext,
		},
		{
			title = ASH_Profiles_UseWhenLeavingGroup,
			name = "ProfileLibLeaveGroupCheckButton",
			type = "checkbutton",
			scripts = scripts.checkbutton,
			overridescripts = {
				OnClick = function() if this:GetChecked() then ProfileLibJoinGroupCheckButton:SetChecked(false) end end,
			},
			tooltiptext = ASH_Profiles_UseWhenLeavingGroup_tooltiptext,
		},
		{
			title = ASH_Profiles_UseOtherwise,
			name = "ProfileLibUseOtherwiseCheckButton",
			type = "checkbutton",
			scripts = scripts.checkbutton,
			tooltiptext = ASH_Profiles_UseOtherwise_tooltiptext,
		},
		{
			title = ASH_Profiles_DefaultProfile,
			name = "ProfileLibDefaultProfileCheckButton",
			type = "checkbutton",
			scripts = scripts.checkbutton,
			tooltiptext = ASH_Profiles_DefaultProfile_tooltiptext,
		},
		{
			title = ASH_Profiles_Create,
			name = "ProfileLibDoneButton",
			type = "button",
			tooltiptext = ASH_Profiles_Create_tooltiptext,
			scripts = {
				OnClick = function() ProfileLib:CreateProfile() end,
			},
		},
		{
			title = ASH_Profiles_Help,
			type = "button",
			tooltiptext = ASH_Profiles_Help_tooltiptext,
			scripts = {
				OnClick = function()
					ProfileLib:Print("|cff00ffffProfile Creator Info:|r")
					ProfileLib:Print("|cff00ff00Basics|r: This menu lets you set profiles of settings for DoTimer, Cooldowns, Communication, and Notifications.  These profiles can be loaded at a later time for quick swapping of multiple settings, or for creating different layouts for different characters.")
					ProfileLib:Print("|cff00ff00Transitions|r: You can select flags for your profiles, which can cause them to be automatically loaded when a certain event occurs, currently entering/leaving combat and joining/leaving a group.")
					ProfileLib:Print("|cff00ff00Rules|r: The addon tries to make the most appropriate selection of which profile to load.  It uses a system of events and conditions.  When you enter/leave combat or join/leave a group, that action is the event.  When that happens, the OTHER pair becomes a condition.")
					ProfileLib:Print("|cff00ff00Continued|r: So if you enter combat and are not grouped, the event is entering combat and the condition is ungrouped.  The addon scans your profiles and selects the ones that meet that event and are not flagged for the opposite of the condition (in the example, grouped).")
					ProfileLib:Print("|cff00ff00Continued|r: Then it separates the character-specific from the general profiles.  It scans the char-spec ones, looking for one that has the correct condition (ungrouped).  If it can't find one, it just looks for one that has no condition (neither grouped nor ungrouped was specified).  Then it looks for an 'Otherwise' profile.")
					ProfileLib:Print("|cff00ff00Continued|r: Then it does the same thing, but for general profiles.  As soon as it finds a profile that matches all this, it stops searching.  Complicated, eh?  Use wisely.")
					ProfileLib:Print("|cff00ff00Continued|r: ======ҰɣξͲˣҪǿѡ˵ãҲûˡ======.")
				end,
			},
		},
		{
			title = ASH_Profiles_Close,
			type = "button",
			tooltiptext = ASH_Profiles_Close_tooltiptext,
			scripts = {
				OnClick = function() this.holder.place.column.menu:Hide() end,
			},
		},
	},
}

function ProfileLib:ShowGUI()
	if not ProfileMenuFrame then
		ProfileLib:CreateGUI(guitable)
	end
	ProfileMenuFrame:Show()
end

local f = CreateFrame("Frame")
f.loaded = false
f.combat = 1
f.grouped = 1
f.potential = {}
f.potentialchar = {}
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PLAYER_REGEN_DISABLED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")
f:RegisterEvent("RAID_ROSTER_UPDATE")
f:RegisterEvent("PARTY_MEMBERS_CHANGED")
f:SetScript("OnEvent",function(self,event) 
	local id = GetRealmName().."-"..UnitName("player")
	if event == "PLAYER_ENTERING_WORLD" then
		if not this.loaded then
			ProfileLib:RegisterSettings() 
			if ProfileLib:Get("profiles") then
				for i,v in ipairs(ProfileLib:Get("profiles")) do
					if not v.flags then v.flags = {} end
					if v.flags["default"] and (v.onlychar and v.onlychar == id) then
						ProfileLib:LoadProfile(v)
						return
					end
				end
				for i,v in ipairs(ProfileLib:Get("profiles")) do
					if v.flags["default"] and not v.onlychar then
						ProfileLib:LoadProfile(v)
						return
					end
				end
			end
		end
	else
		if not ProfileLib:Get("profiles") then return end
		local changed
		if string.find(event,"REGEN") then changed = "combat" else changed = "group" end
		local combat = UnitAffectingCombat("player") and true or false
		local grouped = (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and true or false
		if not (combat == this.combat and grouped == this.grouped) then
			this.combat = combat
			this.grouped = grouped
			--choose the correct profile to load!
			for i,v in ipairs(this.potential) do this.potential[i] = nil end
			for i,v in ipairs(this.potentialchar) do this.potentialchar[i] = nil end
			local combatstr = "combat-"..tostring(combat)
			local wrongcombatstr = "combat-"..tostring(not combat)
			local groupstr = "group-"..tostring(grouped)
			local wronggroupstr = "group-"..tostring(not grouped)
			local primarystr,wrongprimarystr,secondarystr,wrongsecondarystr
			if changed == "combat" then
				primarystr = combatstr
				wrongprimarystr = wrongcombatstr
				secondarystr = groupstr
				wrongsecondarystr = wronggroupstr
			else
				primarystr = groupstr
				wrongprimarystr = wronggroupstr
				secondarystr = combatstr
				wrongsecondarystr = wrongcombatstr
			end
			for _,profile in ipairs(ProfileLib:Get("profiles")) do
				if not profile.flags then profile.flags = {} end
				local flags = profile.flags
				if flags[primarystr] then
					if not flags[wrongsecondarystr] then
						if profile.onlychar then
							if profile.onlychar == id then table.insert(this.potentialchar,profile) end
						else
							table.insert(this.potential,profile)
						end
					end
				end
			end
			if #(this.potentialchar) > 0 then
				for i,v in ipairs(this.potentialchar) do
					if v.flags[secondarystr] then
						ProfileLib:LoadProfile(v)
						return
					end
				end
				ProfileLib:LoadProfile(this.potentialchar[1])
				return
			end
			for i,v in ipairs(ProfileLib:Get("profiles")) do
				if v.flags["otherwise"] and v.onlychar and v.onlychar == id then
					ProfileLib:LoadProfile(v)
					return
				end
			end
			if #(this.potential) > 0 then
				for i,v in ipairs(this.potential) do
					if v.flags[secondarystr] then
						ProfileLib:LoadProfile(v)
						return
					end
				end
				ProfileLib:LoadProfile(this.potential[1])
				return
			end
			for i,v in ipairs(ProfileLib:Get("profiles")) do
				if v.flags["otherwise"] and not v.onlychar then
					ProfileLib:LoadProfile(v)
					return
				end
			end
		end
	end
end)

AsheylaLib:CreateLibrary("ProfileLib",ProfileLib)
