local TimerLibGUI = {}

local scripts = {
	checkbutton = {
		OnShow = function() this:SetChecked(this.self:Get(this.setting)) end,
		OnClick = function() this.self:Set(this.setting,this:GetChecked() and true or false); this.self:UpdateSettings() end,
	},
	slider = {
		OnShow = function() this.real = false this:SetValue(this.self:Get(this.setting)) this.real = true end,
		OnValueChanged = function()
			local value = this:GetValue()
			local formatstring
			if math.floor(value) == value then 
				formatstring = "%d"
			else
				formatstring = "%.2f"
			end
			getglobal(this:GetName().."Value"):SetText(string.format(formatstring,value))
			if this.real then
				this.self:Set(this.setting,value)
				this.self:UpdateSettings()
			end
		end,
	},
	editbox1 = { --initialized with a string
		OnEditFocusGained = function() this:SetText(this.self:Get(this.setting) or ""); this:HighlightText() end,
		OnEditFocusLost = function() this:HighlightText(0,0) end,
		OnShow = function() this:SetText(this.self:Get(this.setting) or "") end,
		OnEscapePressed = function()
			local text = this:GetText()
			if text == "" then
				this:ClearFocus()
			else
				this:SetText("")
			end
		end,
		OnEnterPressed = function() 
			local text = this:GetText()
			this.self:Print("|cff00ffff",this.self:GetName(),"|r:"..ASH_New_setting,text)
			this.self:Set(this.setting,text); this.self:UpdateSettings(); this:ClearFocus()
		end,
	},
	editbox2 = { --controls a table
		OnEditFocusGained = function() this:SetText("") end,
		OnEditFocusLost = function() this:SetText("") end,
		OnEscapePressed = function()
			local text = this:GetText()
			if text == "" then
				this:ClearFocus()
			else
				this.self:Set(this.setting,string.lower(text),nil)
				this.self:Print("|cff00ffff",this.self:GetName(),"|r:"..ASH_Removed_from_list,text)
				this.self:UpdateSettings()				
				this:SetText("")
			end
		end,
		OnEnterPressed = function()
			local text = this:GetText()
			if not (text == "") then
				this.self:Set(this.setting,string.lower(text),1)
				this.self:Print("|cff00ffff",this.self:GetName(),"|r:"..ASH_Added_to_list,text)
				this.self:UpdateSettings()
				this:SetText("")
			end
		end,
		OnEnter = function()
			if this.tooltipText then
				local text = this.tooltipText
				for index,value in pairs(this.self:Get(this.setting)) do
					text = text.."\n"..index
				end
				GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
				GameTooltip:SetText(text, nil, nil, nil, nil, 1);
			end
		end,
	},
	dropdown = {
		OnShow = function()
			getglobal(this:GetName().."Text"):SetText(this.self:Get(this.setting))
		end,
	},
	dropdown2 = {
		OnShow = function()
			getglobal(this:GetName().."Text"):SetText(this.text)
		end,
	},
}

local function dropdown(self,frame,setting,values)
	local info
	for index,value in ipairs(values) do
		info = DropDownLib:GetTable()
		info.text = value
		info.checked = (self:Get(setting) == value) and 1
		info.func = function() 
			self:Set(setting,value) 
			self:UpdateSettings() 
			getglobal(frame:GetName().."Text"):SetText(value)
			DropDownLib:CloseDropDownMenus()
		end
		DropDownLib:AddButton(info,1)
	end
end

local function colordropdown(self)
	local info
	local DROPDOWNLIB_MENU_LEVEL = DropDownLib:MenuLevel()
	local DROPDOWNLIB_MENU_VALUE = DropDownLib:MenuValue()
	if DROPDOWNLIB_MENU_LEVEL == 1 then
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Bar_Text
		local colors = self:Get("bartextcolor")
		info.r = colors.r
		info.g = colors.g
		info.b = colors.b
		info.hasColorSwatch = 1
		local checked
		local currcolor = self:Get("bartextcolor")
		local defcolor = self:GetDefault("bartextcolor")
		if not (currcolor.r == defcolor.r and currcolor.g == defcolor.g and currcolor.b == defcolor.b) then checked = 1 end
		info.checked = checked
		info.func = function() if checked then self:Set("bartextcolor",nil) self:UpdateSettings() end end
		info.swatchFunc = function()
			local r,g,b = DropDownLibColorPicker:GetColorRGB()
			self:Set("bartextcolor","r",r)
			self:Set("bartextcolor","g",g)
			self:Set("bartextcolor","b",b)
			self:UpdateSettings()
		end
		DropDownLib:AddButton(info,1)
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Ghost_Timers
		local colors = self:Get("ghostcolor")
		info.r = colors.r
		info.g = colors.g
		info.b = colors.b
		info.hasColorSwatch = 1
		local checked
		local currcolor = self:Get("ghostcolor")
		local defcolor = self:GetDefault("ghostcolor")
		if not (currcolor.r == defcolor.r and currcolor.g == defcolor.g and currcolor.b == defcolor.b) then checked = 1 end
		info.checked = checked
		info.func = function() if checked then self:Set("ghostcolor",nil) self:UpdateSettings() end end
		info.swatchFunc = function()
			local r,g,b = DropDownLibColorPicker:GetColorRGB()
			self:Set("ghostcolor","r",r)
			self:Set("ghostcolor","g",g)
			self:Set("ghostcolor","b",b)
			self:UpdateSettings()
		end
		DropDownLib:AddButton(info,1)
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Bar_Background
		local colors = self:Get("barbackgroundcolor")
		info.r = colors.r
		info.g = colors.g
		info.b = colors.b
		info.hasColorSwatch = 1
		local checked
		local currcolor = self:Get("barbackgroundcolor")
		local defcolor = self:GetDefault("barbackgroundcolor")
		if not (currcolor.r == defcolor.r and currcolor.g == defcolor.g and currcolor.b == defcolor.b) then checked = 1 end
		info.checked = checked
		info.func = function() if checked then self:Set("barbackgroundcolor",nil) self:UpdateSettings() end end
		info.swatchFunc = function()
			local r,g,b = DropDownLibColorPicker:GetColorRGB()
			self:Set("barbackgroundcolor","r",r)
			self:Set("barbackgroundcolor","g",g)
			self:Set("barbackgroundcolor","b",b)
			self:UpdateSettings()
		end
		DropDownLib:AddButton(info,1)
		info = DropDownLib:GetTable()
		info.text = ASH_COLOR_Target_Name
		local colors = self:Get("targettextcolor")
		info.r = colors.r
		info.g = colors.g
		info.b = colors.b
		info.hasColorSwatch = 1
		local checked
		local currcolor = self:Get("targettextcolor")
		local defcolor = self:GetDefault("targettextcolor")
		if not (currcolor.r == defcolor.r and currcolor.g == defcolor.g and currcolor.b == defcolor.b) then checked = 1 end
		info.checked = checked
		info.func = function() if checked then self:Set("targettextcolor",nil) self:UpdateSettings() end end
		info.swatchFunc = function()
			local r,g,b = DropDownLibColorPicker:GetColorRGB()
			self:Set("targettextcolor","r",r)
			self:Set("targettextcolor","g",g)
			self:Set("targettextcolor","b",b)
			self:UpdateSettings()
		end
		DropDownLib:AddButton(info,1)
	end
end

local function otherdropdown(self)
	local frame = this:GetParent()
	if frame.setting and frame.values then 
		dropdown(self,frame,frame.setting,frame.values) 
	end 
end

local function barmsgtooltip(codes)
	local tooltip = ASH_Bar_Msg_tooltiptext
	for i,v in pairs(codes) do
		tooltip = tooltip..string.format("%s: %s",i,v).."\n"
	end
	tooltip = tooltip..ASH_Bar_Msg_tooltiptext2
	tooltip = tooltip..ASH_Bar_Msg_tooltiptext3
	tooltip = tooltip..ASH_Bar_Msg_tooltiptext4
	tooltip = tooltip..ASH_Bar_Msg_tooltiptext5
	return tooltip
end

local function hiddentypesdropdown(self,orderedtypes,types)
	local info
	if orderedtypes and types then
		for index,value in ipairs(orderedtypes) do
			info = DropDownLib:GetTable()
			local type = string.lower(types[value])
			local checked
			if self:Get("hiddentypes",type) then checked = 1 end
			info.checked = checked
			info.text = value
			info.func = function() if checked then self:Set("hiddentypes",type,nil) else self:Set("hiddentypes",type,1) end self:UpdateSettings() end
			DropDownLib:AddButton(info,1)
		end
	end
end

local noinclude = {}
function TimerLibGUI:SpawnTimerGUI(barmsgformatcodes,orderedtypes,types,...)
	self:ClearTable(noinclude)
	for i = 1,select("#",...) do
		noinclude[(select(i,...))] = 1
	end
	local guitable = {
		title = self:GetName()..ASH_Timer_Settings,
		name = self:GetName().."TimerMenuFrame",
		{ --column 1
			{
				title = ASH_Locked,
				type = "checkbutton",
				tooltiptext = ASH_Locked_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "locked",
				},
				noexists = noinclude["Locked"],
			},
			{
				title = ASH_Headers,
				type = "checkbutton",
				tooltiptext = ASH_Headers_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "names",
				},
				noexists = noinclude["Names"],
			},
			{
				title = ASH_Raid_Icons,
				type = "checkbutton",
				tooltiptext = ASH_Raid_Icons_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "raidicons",
				},
				noexists = noinclude["Raid Icons"],
			},
			{
				title = ASH_Move_Target_Name,
				type = "checkbutton",
				tooltiptext = ASH_Move_Target_Name_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "movetargetname",
				},
				noexists = noinclude["Move Target Name"],
			},
			{
				title = ASH_Clickable,
				type = "checkbutton",
				tooltiptext = ASH_Clickable_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "clickable",
				},
				noexists = noinclude["Clickable"],
			},
			{
				title = ASH_Show_Tenths,
				type = "checkbutton",
				tooltiptext = ASH_Show_Tenths_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "tenths",
				},
				noexists = noinclude["Show Tenths"],
			},
			{
				title = ASH_Icons,
				type = "checkbutton",
				tooltiptext = ASH_Icons_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "icons",
				},
				noexists = noinclude["Icons"],
			},
			{
				title = ASH_Reversed,
				type = "checkbutton",
				tooltiptext = ASH_Reversed_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "reversed",
				},
				noexists = noinclude["Reversed"],
			},
			{
				title = ASH_Expire_Alert,
				type = "checkbutton",
				tooltiptext = ASH_Expire_Alert_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "expalert",
				},
				noexists = noinclude["Expire Alert"],
			},
			{
				title = ASH_Play_Sounds,
				type = "checkbutton",
				tooltiptext = ASH_Play_Sounds_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "playsound",
				},
				noexists = noinclude["Play Sounds"],
			},
			{
				title = ASH_Ghost_Timers,
				type = "checkbutton",
				tooltiptext = ASH_Ghost_Timers_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "ghosts",
				},
				noexists = noinclude["Ghost Timers"],
			},
			{
				title = ASH_Hide_All,
				type = "checkbutton",
				tooltiptext = ASH_Hide_All_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "hideall",
				},
				noexists = noinclude["Hide All"],
			},
			{
				title = ASH_Show_Only_Target,
				type = "checkbutton",
				tooltiptext = ASH_Show_Only_Target_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "onlytarget",
				},
				noexists = noinclude["Show Only Target"],
			},
			{
				title = ASH_Sort_Target,
				type = "checkbutton",
				tooltiptext = ASH_Sort_Target_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "sorttarget",
				},
				noexists = noinclude["Sort Target"],
			},
			{
				title = ASH_SHOW_Only_Focus,
				type = "checkbutton",
				tooltiptext = ASH_SHOW_Only_Focus_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "onlyfocus",
				},
				noexists = noinclude["Show Only Focus"],
			},
			{
				title = ASH_Sort_Focus,
				type = "checkbutton",
				tooltiptext = ASH_Sort_Focus_tooltiptext,
				scripts = scripts.checkbutton,
				data = {
					setting = "sortfocus",
				},
				noexists = noinclude["Sort Focus"],
			},
		},
		{ --column 2
			{
				title = ASH_Scale,
				type = "slider",
				tooltiptext = ASH_Scale_tooltiptext,
				scripts = scripts.slider,
				lowval = .5,
				highval = 2,
				step = .05,
				data = {
					setting = "scale",
				},
				noexists = noinclude["Scale"],
			},
			{
				title = ASH_Timer_Scale,
				type = "slider",
				tooltiptext = ASH_Timer_Scale_tooltiptext,
				scripts = scripts.slider,
				lowval = .5,
				highval = 2,
				step = .05,
				data = {
					setting = "buttonscale",
				},
				noexists = noinclude["Timer Scale"],
			},
			{
				title = ASH_Bar_Length,
				type = "slider",
				tooltiptext = ASH_Bar_Length_tooltiptext,
				scripts = scripts.slider,
				lowval = 50,
				highval = 250,
				step = 5,
				data = {
					setting = "barlength",
				},
				noexists = noinclude["Bar Length"],
			},
			{
				title = ASH_Timer_Spacing,
				type = "slider",
				tooltiptext = ASH_Timer_Spacing_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 20,
				step = 1,
				data = {
					setting = "timerspacing",
				},
				noexists = noinclude["Timer Spacing"],
			},
			{
				title = ASH_Target_Spacing,
				type = "slider",
				tooltiptext = ASH_Target_Spacing_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 30,
				step = 1,
				data = {
					setting = "targetspacing",
				},
				noexists = noinclude["Target Spacing"],
			},
			{
				title = ASH_Alpha,
				type = "slider",
				tooltiptext = ASH_Alpha_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 1,
				step = .05,
				data = {
					setting = "alpha",
				},
				noexists = noinclude["Alpha"],
			},
			{
				title = ASH_Ghost_Duration,
				type = "slider",
				tooltiptext = ASH_Ghost_Duration_tooltiptext,
				scripts = scripts.slider,
				lowval = 1,
				highval = 30,
				step = 1,
				data = {
					setting = "ghostdata",
				},
				noexists = noinclude["Ghost Duration"],
			},
			{
				title = ASH_Bar_Texture,
				type = "slider",
				tooltiptext = ASH_Bar_Texture_tooltiptext,
				scripts = scripts.slider,
				lowval = 1,
				highval = 15,
				step = 1,
				data = {
					setting = "bartexture",
				},
				noexists = noinclude["Bar Texture"],
			},
			{
				title = ASH_Fade_In_Time,
				type = "slider",
				tooltiptext = ASH_Fade_In_Time_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 5,
				step = 1,
				data = {
					setting = "fadeintime",
				},
				noexists = noinclude["Fade In Time"],
			},
			{
				title = ASH_Fade_Out_Time,
				type = "slider",
				tooltiptext = ASH_Fade_Out_Time_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 5,
				step = 1,
				data = {
					setting = "fadeouttime",
				},
				noexists = noinclude["Fade Out Time"],
			},
			{
				title = ASH_Max_Targets,
				type = "slider",
				tooltiptext = ASH_Max_Targets_tooltiptext,
				scripts = scripts.slider,
				lowval = 0,
				highval = 10,
				step = 1,
				data = {
					setting = "maxtargets",
				},
				noexists = noinclude["Max Targets"],
			},
			{
				title = ASH_Bar_Msg,
				type = "editbox",
				tooltiptext = barmsgtooltip(barmsgformatcodes),
				scripts = scripts.editbox1,
				data = {
					setting = "barmsg",
				},
				noexists = noinclude["Bar Msg"],
			},
			{
				title = ASH_Hidden_Timers,
				type = "editbox",
				tooltiptext = ASH_Hidden_Timers_tooltiptext,
				scripts = scripts.editbox2,
				data = {
					setting = "hidden",
				},
				noexists = noinclude["Hidden Timers"],
			},
		},
		{ --column 3
			{
				title = ASH_Reset_Position,
				type = "button",
				tooltiptext = ASH_Reset_Position_tooltiptext,
				buffer = 10,
				scripts = {
					OnClick = function() this.self:Set("offsetX",nil); this.self:UpdateSettings() end,
				},
				noexists = noinclude["Reset Position"],
			},
			{
				title = ASH_Show_Preview,
				type = "button",
				tooltiptext = ASH_Show_Preview_tooltiptext,
				buffer = 10,
				scripts = {
					OnClick = function() this.self:ShowTimers(3,4) end,
				},
				noexists = noinclude["Show Preview"],
			},
			{
				title = ASH_Hide_Preview,
				type = "button",
				tooltiptext = ASH_Hide_Preview_tooltiptext,
				buffer = 10,
				scripts = {
					OnClick = function() this.self:RemoveAllTimers() end,
				},
				noexists = noinclude["Hide Preview"],
			},
			{
				title = ASH_Timer_Format,
				type = "dropdown",
				tooltiptext = ASH_Timer_Format_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "format",
					values = {ASH_bars,ASH_icon,ASH_text},
				},
				noexists = noinclude["Timer Format"],
			},
			{
				title = ASH_Target_Layout,
				type = "dropdown",
				tooltiptext = ASH_Target_Layout_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "targetlayout",
					values = {ASH_up,ASH_down,ASH_left,ASH_right},
				},
				noexists = noinclude["Target Layout"],
			},
			{
				title = ASH_Timer_Layout,
				type = "dropdown",
				tooltiptext = ASH_Timer_Layout_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "timerlayout",
					values = {ASH_up,ASH_down,ASH_left,ASH_right},
				},
				noexists = noinclude["Timer Layout"],
			},
			{
				title = ASH_Sorting_Method,
				type = "dropdown",
				tooltiptext = ASH_Sorting_Method_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "sortmethod",
					values = {ASH_added_A,ASH_added_D,ASH_remaining_A,ASH_remaining_D,ASH_alphabetical_A,ASH_alphabetical_D},
				},
				noexists = noinclude["Sorting Method"],
			},
			{
				title = ASH_Target_Sorting,
				type = "dropdown",
				tooltiptext = ASH_Target_Sorting_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "targetsort",
					values = {ASH_added_A,ASH_added_D,ASH_alphabetical_A,ASH_alphabetical_D},
				},
				noexists = noinclude["Target Sorting"],
			},
			{
				title = ASH_Seconds_Format,
				type = "dropdown",
				tooltiptext = ASH_Seconds_Format_tooltiptext,
				func = function() otherdropdown(self) end,
				scripts = scripts.dropdown,
				data = {
					setting = "secondsformat",
					values = {"##",":##","0:##","##s","##s abbv"},
				},
				noexists = noinclude["Seconds Format"],
			},
			{
				title = ASH_Colors,
				type = "dropdown",
				tooltiptext = ASH_Colors_tooltiptext,
				func = function() colordropdown(self) end,
				scripts = scripts.dropdown2,
				data = {
					text = ASH_Choose_text,
				},
				noexists = noinclude["Colors"],
			},
			{
				title = ASH_Hidden_Types,
				type = "dropdown",
				tooltiptext = ASH_Hidden_Types_tooltiptext,
				func = function() hiddentypesdropdown(self,orderedtypes,types) end,
				scripts = scripts.dropdown2,
				data = {
					text = ASH_Choose_text,
				},
				noexists = noinclude["Hidden Types"],
			},
			"BREAK",
			{
				title = ASH_Return,
				type = "button",
				tooltiptext = ASH_Return_tooltiptext,
				scripts = {
					OnClick = function() this.self:ShowGUI(); end,
				},
				noexists = noinclude["Return"],
			},
			{
				title = ASH_Close,
				type = "button",
				tooltiptext = ASH_Close_tooltiptext,
				scripts = {
					OnClick = function() this.holder.place.column.menu:Hide() end,
				},
				noexists = noinclude["Close"],
			},
		},
	}
	return guitable
end

function TimerLibGUI:ShowTimerGUI(barmsgformatcodes,orderedtypes,types,...)
	if not getglobal(self:GetName().."TimerMenuFrame") then
		self:CreateGUI(self:SpawnTimerGUI(barmsgformatcodes,orderedtypes,types,...))
	end
	getglobal(self:GetName().."TimerMenuFrame"):Show()
end

AsheylaLib:CreateLibrary("TimerLibGUI",TimerLibGUI)
