
MOVABLEBAR_MAXNUM = 10;
--------------------------------------------------
function MovableButton_OnLoad()
	this.showgrid = 0;
	this.flashing = 0;
	this.flashtime = 0;
	this.buttonType = "MULTIACTIONBAR1BUTTON"
	MovableButton_Update();
	this:SetAttribute("type", "action");
	this:SetAttribute("checkselfcast", true);
	this:SetAttribute("useparent-unit", true);
	this:RegisterForDrag("LeftButton", "RightButton");
	this:RegisterForClicks("AnyUp");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UPDATE_BONUS_ACTIONBAR");
	this:RegisterEvent("ACTIONBAR_SHOWGRID");
	this:RegisterEvent("ACTIONBAR_HIDEGRID");
	this:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	this:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	this:RegisterEvent("UPDATE_BINDINGS");

	MovableButton_UpdateHotkeys(this.buttonType);
end

function MovableButton_UpdateHotkeys()
    local actionButtonType = "MOVABLEBAR"..this:GetParent():GetID().."BUTTON";

    local hotkey = getglobal(this:GetName().."HotKey");
    local action = MovableButton_GetPagedID(this);
    local key = GetBindingKey(actionButtonType..this:GetID()) or
                GetBindingKey("CLICK "..this:GetName()..":LeftButton");

    if ( GetBindingText(key, "KEY_", 1) == "" ) then
        if ( not HasAction(action) ) then
            hotkey:SetText("");
        elseif ( ActionHasRange(action) ) then
            if ( IsActionInRange(action) ) then
                hotkey:SetText(RANGE_INDICATOR);
                hotkey:SetTextHeight(8);
                hotkey:SetPoint("TOPRIGHT", this:GetName(), "TOPRIGHT", -3, 5);
            else
                hotkey:SetText("");
            end
        end
    else
        hotkey:SetText(GetBindingText(key, "KEY_", 1));
    end
end

function MovableButton_Update()
	-- Special case code for bonus bar buttons
	-- Prevents the button from updating if the bonusbar is still in an animation transition
	if ( this.isBonus and this.inTransition ) then
		MovableButton_UpdateUsable();
		MovableButton_UpdateCooldown();
		return;
	end

	local action = MovableButton_GetPagedID(this);
	local icon = getglobal(this:GetName().."Icon");
	local buttonCooldown = getglobal(this:GetName().."Cooldown");
	local texture = GetActionTexture(action);
	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		this.rangeTimer = -1;
		this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
		-- Save texture if the button is a bonus button, will be needed later
		if ( this.isBonus ) then
			this.texture = texture;
		end
	else
		icon:Hide();
		buttonCooldown:Hide();
		this.rangeTimer = nil;
		this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
		getglobal(this:GetName().."HotKey"):SetVertexColor(0.6, 0.6, 0.6);
	end
	MovableButton_UpdateCount();
	if ( HasAction(action) ) then
		this:RegisterEvent("ACTIONBAR_UPDATE_STATE");
		this:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
		this:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
		this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
		this:RegisterEvent("PLAYER_AURAS_CHANGED");
		this:RegisterEvent("PLAYER_TARGET_CHANGED");
		this:RegisterEvent("UNIT_INVENTORY_CHANGED");
		this:RegisterEvent("CRAFT_SHOW");
		this:RegisterEvent("CRAFT_CLOSE");
		this:RegisterEvent("TRADE_SKILL_SHOW");
		this:RegisterEvent("TRADE_SKILL_CLOSE");
		this:RegisterEvent("PLAYER_ENTER_COMBAT");
		this:RegisterEvent("PLAYER_LEAVE_COMBAT");
		this:RegisterEvent("START_AUTOREPEAT_SPELL");
		this:RegisterEvent("STOP_AUTOREPEAT_SPELL");

		if ( not this:GetAttribute("statehidden") and not this.wshide and not InCombatLockdown()) then
			this:Show();
		end
		MovableButton_UpdateState();
		MovableButton_UpdateUsable();
		MovableButton_UpdateCooldown();
		MovableButton_UpdateFlash();
	else
		this:UnregisterEvent("ACTIONBAR_UPDATE_STATE");
		this:UnregisterEvent("ACTIONBAR_UPDATE_USABLE");
		this:UnregisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
		this:UnregisterEvent("UPDATE_INVENTORY_ALERTS");
		this:UnregisterEvent("PLAYER_AURAS_CHANGED");
		this:UnregisterEvent("PLAYER_TARGET_CHANGED");
		this:UnregisterEvent("UNIT_INVENTORY_CHANGED");
		this:UnregisterEvent("CRAFT_SHOW");
		this:UnregisterEvent("CRAFT_CLOSE");
		this:UnregisterEvent("TRADE_SKILL_SHOW");
		this:UnregisterEvent("TRADE_SKILL_CLOSE");
		this:UnregisterEvent("PLAYER_ENTER_COMBAT");
		this:UnregisterEvent("PLAYER_LEAVE_COMBAT");
		this:UnregisterEvent("START_AUTOREPEAT_SPELL");
		this:UnregisterEvent("STOP_AUTOREPEAT_SPELL");

		if ( this.showgrid == 0 ) then
			this:Hide();
		else
			buttonCooldown:Hide();
		end
	end

	-- Add a green border if button is an equipped item
	local border = getglobal(this:GetName().."Border");
	if ( IsEquippedAction(action) ) then
		border:SetVertexColor(0, 1.0, 0, 0.35);
		border:Show();
	else
		border:Hide();
	end

	if ( GameTooltip:IsOwned(this) ) then
		MovableButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end

	-- Update Macro Text
	local macroName = getglobal(this:GetName().."Name");
	if ( not IsConsumableAction(action) and not IsStackableAction(action) ) then
		macroName:SetText(GetActionText(action));
	else
		macroName:SetText("");
	end

	-- Save the action so we don't update in OnUpdate()
	this.action = action;

	if(not InCombatLockdown()) then
	this:SetID(action) 
	end
end

function MovableButton_ShowGrid(button)
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid+1;
	getglobal(button:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);

	if ( not button:GetAttribute("statehidden") and not button.wshide  and not InCombatLockdown()) then
		button:Show();
	end
end

function MovableButton_HideGrid(button)	
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid-1;
	if ( button.showgrid == 0 and not HasAction(MovableButton_GetPagedID(button)) ) then
		button:Hide();
	end
end

function MovableButton_UpdateState(button)
	if ( not button ) then
		button = this;
	end
	local actionID = MovableButton_GetPagedID(button);
	if ( IsCurrentAction(actionID) or IsAutoRepeatAction(actionID) ) then
		button:SetChecked(1);
	else
		button:SetChecked(0);
	end
end

function MovableButton_UpdateUsable()
	local icon = getglobal(this:GetName().."Icon");
	local normalTexture = getglobal(this:GetName().."NormalTexture");
	local isUsable, notEnoughMana = IsUsableAction(MovableButton_GetPagedID(this));
	if ( isUsable ) then
		icon:SetVertexColor(1.0, 1.0, 1.0);
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
	elseif ( notEnoughMana ) then
		icon:SetVertexColor(0.5, 0.5, 1.0);
		normalTexture:SetVertexColor(0.5, 0.5, 1.0);
	else
		icon:SetVertexColor(0.4, 0.4, 0.4);
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
	end

	if ( IsActionInRange( MovableButton_GetPagedID(this)) == 0 ) then
		icon:SetVertexColor(1.0, 0.1, 0.1);
	end
end

function MovableButton_UpdateCount()
	local text = getglobal(this:GetName().."Count");
	local action = MovableButton_GetPagedID(this);
	if ( IsConsumableAction(action) or IsStackableAction(action) ) then
		text:SetText(GetActionCount(action));
	else
		text:SetText("");
	end
end

function MovableButton_UpdateCooldown()
	local cooldown = getglobal(this:GetName().."Cooldown");
	local start, duration, enable = GetActionCooldown(MovableButton_GetPagedID(this));
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
end

function MovableButton_OnEvent(event)
	if ( event == "ACTIONBAR_SLOT_CHANGED" ) then
		if ( arg1 == 0 or arg1 == MovableButton_GetPagedID(this) ) then
			MovableButton_Update();
		end
		return;
	end
	if ( event == "PLAYER_ENTERING_WORLD" or event == "ACTIONBAR_PAGE_CHANGED" ) then
		MovableButton_Update();
		return;
	end
	if ( event == "UPDATE_BONUS_ACTIONBAR" ) then
		if ( this.isBonus ) then
			MovableButton_Update();
		end
		return;
	end
	if ( event == "ACTIONBAR_SHOWGRID" ) then
		MovableButton_ShowGrid();
		return;
	end
	if ( event == "ACTIONBAR_HIDEGRID" ) then
		MovableButton_HideGrid();
		return;
	end
	if ( event == "UPDATE_BINDINGS" ) then
		MovableButton_UpdateHotkeys(this.buttonType);
		return;
	end

	-- All event handlers below this line are only set when the button has an action

	if ( event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_AURAS_CHANGED" ) then
		MovableButton_UpdateUsable();
		MovableButton_UpdateHotkeys(this.buttonType);
	elseif ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == "player" ) then
			MovableButton_Update();
		end
	elseif ( event == "ACTIONBAR_UPDATE_STATE" ) then
		MovableButton_UpdateState();
	elseif ( event == "ACTIONBAR_UPDATE_USABLE" or event == "UPDATE_INVENTORY_ALERTS" or event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		MovableButton_UpdateUsable();
		MovableButton_UpdateCooldown();
	elseif ( event == "CRAFT_SHOW" or event == "CRAFT_CLOSE" or event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" ) then
		MovableButton_UpdateState();
	elseif ( event == "PLAYER_ENTER_COMBAT" ) then
		if ( IsAttackAction(MovableButton_GetPagedID(this)) ) then
			MovableButton_StartFlash();
		end
	elseif ( event == "PLAYER_LEAVE_COMBAT" ) then
		if ( IsAttackAction(MovableButton_GetPagedID(this)) ) then
			MovableButton_StopFlash();
		end
	elseif ( event == "START_AUTOREPEAT_SPELL" ) then
		if ( IsAutoRepeatAction(MovableButton_GetPagedID(this)) ) then
			MovableButton_StartFlash();
		end
	elseif ( event == "STOP_AUTOREPEAT_SPELL" ) then
		if ( MovableButton_IsFlashing() and not IsAttackAction(MovableButton_GetPagedID(this)) ) then
			MovableButton_StopFlash();
		end
	end
end

function MovableButton_SetTooltip()
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, this);
	else
		if ( this:GetParent() == MultiBarBottomRight or this:GetParent() == MultiBarRight or this:GetParent() == MultiBarLeft ) then
			GameTooltip:SetOwner(this, "ANCHOR_LEFT");
		else
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		end
		
	end
	
	if ( GameTooltip:SetAction(MovableButton_GetPagedID(this)) ) then
		this.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		this.updateTooltip = nil;
	end
end

function MovableButton_OnUpdate(elapsed)

	local hotkey = getglobal(this:GetName().."HotKey");
	local action = MovableButton_GetPagedID(this);

	if ( this.action ~= action ) then
		MovableButton_Update();
	end

	if ( hotkey:GetText() == RANGE_INDICATOR ) then
		if ( IsActionInRange(action) == 1 ) then
			hotkey:Hide();
		else
			hotkey:Show();
		end
	end

	if ( MovableButton_IsFlashing() ) then
		this.flashtime = this.flashtime - elapsed;
		if ( this.flashtime <= 0 ) then
			local overtime = -this.flashtime;
			if ( overtime >= ATTACK_BUTTON_FLASH_TIME ) then
				overtime = 0;
			end
			this.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime;

			local flashTexture = getglobal(this:GetName().."Flash");
			if ( flashTexture:IsShown() ) then
				flashTexture:Hide();
			else
				flashTexture:Show();
			end
		end
	end
	
	-- Handle range indicator
	if ( this.rangeTimer ) then
		this.rangeTimer = this.rangeTimer - elapsed;

		if ( this.rangeTimer <= 0 ) then
			MovableButton_UpdateUsable();
			local count = getglobal(this:GetName().."HotKey");
			if ( IsActionInRange(action) == 0 ) then
				count:SetVertexColor(1.0, 0.1, 0.1);
			else
				count:SetVertexColor(0.6, 0.6, 0.6);
			end
			this.rangeTimer = TOOLTIP_UPDATE_TIME;
		end
	end

	if ( not this.updateTooltip ) then
		return;
	end

	this.updateTooltip = this.updateTooltip - elapsed;
	if ( this.updateTooltip > 0 ) then
		return;
	end

	if ( GameTooltip:IsOwned(this) ) then
		MovableButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end
end

function MovableButton_GetPagedID(button)
	local id = button:GetParent():GetID();
	local name = button:GetName();
	local _,_,_,num = string.find(name,"(.+)ActionButton(.+)")
	local num = tonumber(num)
	if(not WS_MovableBar) then
		WS_MovableBar_Init();
	end
	if(InCombatLockdown()) then
		local newid = 12*(WS_MovableBar["Option"][id]["Num"]-1) + num + (GetActionBarPage() - 1) * NUM_ACTIONBAR_BUTTONS;
		if(newid > 120) then
			newid = newid - 120;
		end
		return newid;
	else
		return 12*(WS_MovableBar["Option"][id]["Num"]-1) + num;
	end
end

function MovableButton_UpdateFlash()
	local action = MovableButton_GetPagedID(this);
	if ( (IsAttackAction(action) and IsCurrentAction(action)) or IsAutoRepeatAction(action) ) then
		MovableButton_StartFlash();
	else
		MovableButton_StopFlash();
	end
end

function MovableButton_StartFlash()
	this.flashing = 1;
	this.flashtime = 0;
	MovableButton_UpdateState();
end

function MovableButton_StopFlash()
	this.flashing = 0;
	getglobal(this:GetName().."Flash"):Hide();
	MovableButton_UpdateState();
end

function MovableButton_IsFlashing()
	if ( this.flashing == 1 ) then
		return 1;
	else
		return nil;
	end
end

function MovableBarFrameDragButton_OnClick(id,arg1)
	if ( arg1 == "RightButton" ) then
		if(WS_MovableBar["Option"][id]["state"] == 1) then
			WS_MovableBar["Option"][id]["state"] = 2;
			MovableBarFrameChangeState(id , WS_MovableBar["Option"][id].state);
			MovableBar_Saveoptions();
			return;
		elseif(WS_MovableBar["Option"][id]["state"] == 2) then
			WS_MovableBar["Option"][id]["state"] = 3;
			MovableBarFrameChangeState(id , WS_MovableBar["Option"][id].state);
			MovableBar_Saveoptions();
			return;
		elseif(WS_MovableBar["Option"][id]["state"] == 3) then
			WS_MovableBar["Option"][id]["state"] = 1;
			MovableBarFrameChangeState(id , WS_MovableBar["Option"][id].state);
			MovableBar_Saveoptions();
			return;
		end
	end	
end

function MovableBarFrameChangeState(id , stateId)
	if(stateId == 1) then
		local mainButton = getglobal("MovableBarFrame"..id.."ActionButton1");
		for i = 2 , 12 , 1 do
			local button = getglobal("MovableBarFrame"..id.."ActionButton"..i);
			button:ClearAllPoints();
			button:SetPoint("BOTTOMLEFT",mainButton,"BOTTOMRIGHT",5,0);			
			mainButton = button:GetName();
		end
	elseif(stateId == 2) then
		local mainButton = getglobal("MovableBarFrame"..id.."ActionButton1");
		for i = 2 , 12 , 1 do
			local button = getglobal("MovableBarFrame"..id.."ActionButton"..i);
			button:ClearAllPoints();
			button:SetPoint("TOP",mainButton,"BOTTOM",0,-5);
			mainButton = button:GetName();
		end
	elseif(stateId == 3) then
		local mainButton = getglobal("MovableBarFrame"..id.."ActionButton1");
		for i = 2 , 12 , 1 do
			local button = getglobal("MovableBarFrame"..id.."ActionButton"..i);
			button:ClearAllPoints();
			if(i == 4 or i == 7 or i == 10) then
				local n = i-3;
				mainButton = getglobal("MovableBarFrame"..id.."ActionButton"..n);
				button:SetPoint("TOPLEFT",mainButton,"BOTTOMLEFT",0,-5);
			else
				button:SetPoint("BOTTOMLEFT",mainButton,"BOTTOMRIGHT",5,0);
			end
			mainButton = button:GetName();
		end
	end
end

function MovableBarFrameDragButton_OnEnter()
	local parent = this:GetParent():GetName();
	local barId = this:GetParent():GetID();
	getglobal(parent.."RightButton"):Show();
	getglobal(parent.."LeftButton"):Show();
	getglobal(this:GetName().."BarNum"):Show();
	getglobal(this:GetName().."BarNum"):SetText(WS_MovableBar["Option"][barId]["Num"]);
end

function MovableBarFrameDragButton_OnLeave()
	local parent = this:GetParent():GetName();
	getglobal(parent.."RightButton"):Hide();
	getglobal(parent.."LeftButton"):Hide();
	getglobal(parent.."DragButtonBarNum"):Hide();
end

function MovableBarFrameDragButton_OnLeft(barId)
	if(not InCombatLockdown()) then
		if(WS_MovableBar["Option"][barId]["Num"] > 1) then
			WS_MovableBar["Option"][barId]["Num"] = WS_MovableBar["Option"][barId]["Num"] - 1;
		elseif(WS_MovableBar["Option"][barId]["Num"] == 1) then
			WS_MovableBar["Option"][barId]["Num"] = MOVABLEBAR_MAXNUM;
		end
		MovableBar_Saveoptions();
		getglobal(this:GetParent():GetName().."DragButtonBarNum"):SetText(WS_MovableBar["Option"][barId]["Num"]);
		MovableButton_ResetBar();
	end
end

function MovableBarFrameDragButton_OnRight(barId)
	if(not InCombatLockdown()) then
		if(WS_MovableBar["Option"][barId]["Num"] < MOVABLEBAR_MAXNUM) then
			WS_MovableBar["Option"][barId]["Num"] = WS_MovableBar["Option"][barId]["Num"] + 1;
		elseif(WS_MovableBar["Option"][barId]["Num"] == MOVABLEBAR_MAXNUM) then
			WS_MovableBar["Option"][barId]["Num"] = 1;
		end

		MovableBar_Saveoptions();
		getglobal(this:GetParent():GetName().."DragButtonBarNum"):SetText(WS_MovableBar["Option"][barId]["Num"]);
		MovableButton_ResetBar();
	end
end

function MovableButton_ResetBar()
	ActionBar_PageUp();
	ActionBar_PageDown();
end

function MovableBar_UpdateButtonScale(frameId)
	for i = 1, 12, 1 do
		if(i <= WS_MovableBar["Option"][frameId].ScaleButton) then
			getglobal("MovableBarFrame"..frameId.."ActionButton"..i):SetScale(WS_MovableBar["Option"][frameId].Scale/100);
		else
			getglobal("MovableBarFrame"..frameId.."ActionButton"..i):SetScale(1);
		end
	end
end


function MovableBarFrame_OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
end

function MovableBarFrame_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		ToggleMovableBarNums(WS_MovableBar.Bars);
		if(WS_MovableBar and WS_MovableBar.AutoHide and WS_MovableBar.AutoHide == "1") then
			for i = 1, 4 , 1 do
				getglobal("MovableBarFrame"..i):Hide();
			end		
		end
		for i = 1, 4 , 1 do
			ResetMovableBarButtons(i);
			MovableBar_UpdateButtonScale(i);
			MovableBarFrameChangeState(i,WS_MovableBar["Option"][i].state);
		end
		
		local id = this:GetID();
		local barNum = getglobal(this:GetName().."DragButtonBarNum");
		if(id == 1) then
			barNum:SetTextColor(1.0, 1.0, 1.0);
		elseif(id == 2) then
			barNum:SetTextColor(0.5, 0.5, 1.0);
		elseif(id == 3) then			
			barNum:SetTextColor(0.3, 1.0, 0.3);
		elseif(id == 4) then
			barNum:SetTextColor(1.0, 0, 0);
		end	
		return;
	end
	if(event == "PLAYER_REGEN_ENABLED" and WS_MovableBar.AutoHide == "1") then
		for i = 1, 4 , 1 do
			getglobal("MovableBarFrame"..i):Hide();
		end
		return;
	end
	if(event == "PLAYER_REGEN_DISABLED" and WS_MovableBar.AutoHide == "1") then
		ToggleMovableBarNums(WS_MovableBar.Bars);
		return;
	end
end
function MovableButton_HotkeyClick(barid, id)
	if ( keystate == "down" ) then
		MultiActionButtonDown("MovableBarFrame"..barid, id)
	else
		MultiActionButtonUp("MovableBarFrame"..barid, id, onSelf)
	end
end
