local PartyQuestComm_find1 = gsub(TEXT(ERR_QUEST_ADD_FOUND_SII), "%%s(.+)%%d(.+)%%d", "(.+)%1%%d+%2%%d+")
local PartyQuestComm_find2 = "%[%d+%].+%(.+%)"

local PartyQuestComm_version = "20060919"
local PartyQuestComm_Checked = { }
local PartyQuestComm_inited
local PartyQuestComm_Prefix = { progress = "PQC_Progress", complete = "PQC_complete", fail = "PQC_fail", sync = "PQC_sync" }

function PartyQuestComm_OnLoad()
	SlashCmdList["PARTYQUESTCOMMCOMMAND"] = PartyQuestComm_SlashHandler
	SLASH_PARTYQUESTCOMMCOMMAND1 = "/partyquestcomm"
	SLASH_PARTYQUESTCOMMCOMMAND2 = "/pqc"

	this:RegisterEvent("VARIABLES_LOADED")
	this:RegisterEvent("CHAT_MSG_ADDON")
	this:RegisterEvent("UI_INFO_MESSAGE")
	this:RegisterEvent("QUEST_LOG_UPDATE")
end

function PartyQuestComm_Options_Reset()
	PartyQuestComm_Options						= {}
	PartyQuestComm_Options.toggle				= 1
	PartyQuestComm_Options.DisplayUIErrorFrame	= 1
	PartyQuestComm_Options.DisplayChatFrame		= 0
	PartyQuestComm_Options.version				= PartyQuestComm_version
end

function PartyQuestComm_Init (force)
	PartyQuestComm_Checked = { }
	PartyQuestComm_inited = nil
	local numEntries = GetNumQuestLogEntries()
	if (not numEntries or numEntries == 0) and not force then
		return
	end
	for i = 1, numEntries do
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(i)
		if (level and not isHeader and isComplete and isComplete == 1) then
			PartyQuestComm_Checked[questLogTitleText] = 1
		end
	end
	PartyQuestComm_inited = true
end

function PartyQuestComm_IsInParty(name)
	for i=1, MAX_PARTY_MEMBERS, 1 do
		if (UnitName("party"..i) == name) then
			return true
		end
	end
	return
end

function PartyQuestComm_SlashHandler(msg)
	msg = string.lower(msg)
	if (msg == "on") then
		PartyQuestComm_Options.toggle = 1
		UIErrorsFrame:AddMessage("PartyQuestComm: 启用", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME)
		DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 启用")
	elseif (msg == "off") then
		PartyQuestComm_Options.toggle = 0
		UIErrorsFrame:AddMessage("PartyQuestComm: 禁用", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME)
		DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 禁用")
	elseif (msg == "reset") then
		PartyQuestComm_Options_Reset()
		PartyQuestComm_Init(true)
		DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 重置")
	elseif (msg == "uierr") then
		if (PartyQuestComm_Options.DisplayUIErrorFrame ~= 1) then
			PartyQuestComm_Options.DisplayUIErrorFrame = 1
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 主画面信息显示开启")
		else
			PartyQuestComm_Options.DisplayUIErrorFrame = 0
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 主画面信息显示关闭")
		end
	elseif (msg == "chat") then
		if (PartyQuestComm_Options.DisplayChatFrame ~= 1) then
			PartyQuestComm_Options.DisplayChatFrame = 1
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 聊天框信息显示开启")
		else
			PartyQuestComm_Options.DisplayChatFrame = 0
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 聊天框信息显示关闭")
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: "..PartyQuestComm_version)
		if (PartyQuestComm_Options.toggle == 1) then
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 启用")
		else
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 禁用")
		end
		if (PartyQuestComm_Options.DisplayUIErrorFrame == 1) then
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 主画面信息显示启用")
		else
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 主画面信息显示禁用")
		end
		if (PartyQuestComm_Options.DisplayChatFrame == 1) then
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 聊天框信息显示启用")
		else
			DEFAULT_CHAT_FRAME:AddMessage("PartyQuestComm: 聊天框信息显示禁用")
		end
		DEFAULT_CHAT_FRAME:AddMessage("命令: /pqc on | off | reset | chat | uierr")
	end
end

function PartyQuestComm_OnEvent(event)
	if (event=="VARIABLES_LOADED") then
		if (type(PartyQuestComm_Options)~="table" or not PartyQuestComm_Options.toggle or not PartyQuestComm_Options.DisplayUIErrorFrame or not PartyQuestComm_Options.DisplayChatFrame or not PartyQuestComm_Options.version or PartyQuestComm_Options.version ~= PartyQuestComm_version) then
			PartyQuestComm_Options_Reset()
		end
		PartyQuestComm_Init()
	elseif (PartyQuestComm_Options.toggle == 1) then
		if (event == "QUEST_LOG_UPDATE") then
			if not PartyQuestComm_inited then
				return PartyQuestComm_Init(true)
			end
			local numEntries = GetNumQuestLogEntries()
			for i = 1, numEntries do
				local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle( i )
				if (level and not isHeader and isComplete) then
					if isComplete == 1 then 
						if not PartyQuestComm_Checked[questLogTitleText] then
							PartyQuestComm_Checked[questLogTitleText] = 1
							if (DEFAULT_CHAT_FRAME and PartyQuestComm_Options.DisplayChatFrame == 1) then
								DEFAULT_CHAT_FRAME:AddMessage("["..level.."] "..questLogTitleText.." (完成)")
							end
							if (GetNumPartyMembers() > 0) then
								SendAddonMessage(PartyQuestComm_Prefix.complete, "["..level.."] "..questLogTitleText.." (完成)", "PARTY")
							end
						end
					elseif isComplete == -1 then
						if (DEFAULT_CHAT_FRAME and PartyQuestComm_Options.DisplayChatFrame == 1) then
							DEFAULT_CHAT_FRAME:AddMessage("["..level.."] "..questLogTitleText.." (失败)")
						end
						if (GetNumPartyMembers() > 0) then
							SendAddonMessage(PartyQuestComm_Prefix.fail, "["..level.."] "..questLogTitleText.." (失败)", "PARTY")
						end
					end
				end
			end
		elseif (event == "UI_INFO_MESSAGE" and PartyQuestComm_inited) then
			if strfind(arg1, PartyQuestComm_find1) and GetNumPartyMembers() > 0 then
				SendAddonMessage(PartyQuestComm_Prefix.progress, arg1, "PARTY")
			end
		elseif (event == "CHAT_MSG_ADDON") then
			PartyQuestComm_FromParty(arg1, arg2, arg3, arg4)
		end
	end
end

function PartyQuestComm_FromParty(prefix, message, dist, sender)
	if (PartyQuestComm_Options.toggle == 1) then
		if (sender == UnitName("player") or dist ~= "PARTY" or not PartyQuestComm_IsInParty(sender) or not message) then return end

		if (
			(prefix == PartyQuestComm_Prefix.progress and strfind(message, PartyQuestComm_find1)) or
			((prefix == PartyQuestComm_Prefix.complete or prefix == PartyQuestComm_Prefix.fail) and strfind(message, PartyQuestComm_find2))
		) then

			if (PartyQuestComm_Options.DisplayUIErrorFrame == 1) then
				UIErrorsFrame:AddMessage("["..sender.."] "..message, 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME)
			end
			if (PartyQuestComm_Options.DisplayChatFrame == 1) then 
				DEFAULT_CHAT_FRAME:AddMessage("["..sender.."] "..message)
			end
		end
	end
end

function TogglePartyQuest (flag)
	if(not PartyQuestComm) then
		PartyQuestComm = {};
	end
	if(flag == "1") then
		PartyQuestComm.toggle = 1;
	else
		PartyQuestComm.toggle = 0;
	end
end