
local wsPopBarFrameShowHidden = 0;
local wsPopBarFrameLastUpdateTime = 0;
local WSPOPBARFRAME_UPDATEINTERVAL = 0.2;

function wsPopBarButton_GetBindButton(row, col)
	local retid = string.format("wsPopBarButton%d%02d", row, col);
	return retid;
end

function wsPopBarButton_OnLoad()
	this.showgrid = 0;
	this.flashing = 0;
	this.flashtime = 0;
	wsPopBarButton_Update();
	this:SetAttribute("type", "action");
	this:SetAttribute("checkselfcast", true);
	this:SetAttribute("useparent-unit", true);
	this:RegisterForDrag("LeftButton", "RightButton");
	this:RegisterForClicks("AnyUp");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UPDATE_BONUS_ACTIONBAR");
	this:RegisterEvent("ACTIONBAR_SHOWGRID");
	this:RegisterEvent("ACTIONBAR_HIDEGRID");
	this:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	this:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	this:RegisterEvent("UPDATE_BINDINGS");

	wsPopBarButton_UpdateHotkeys(this.buttonType);
end

function wsPopBarButton_UpdateHotkeys()
    local hotkey = getglobal(this:GetName().."HotKey");
    local action = wsPopBarButton_GetPagedID(this);
    local key = GetBindingKey("CLICK "..this:GetName()..":LeftButton");

    if ( GetBindingText(key, "KEY_", 1) == "" ) then
        if ( not HasAction(action) ) then
            hotkey:SetText("");
        elseif ( ActionHasRange(action) ) then
            if ( IsActionInRange(action) ) then
                hotkey:SetText(RANGE_INDICATOR);
                hotkey:SetTextHeight(8);
                hotkey:SetPoint("TOPRIGHT", this:GetName(), "TOPRIGHT", -3, 5);
            else
                hotkey:SetText("");
            end
        end
    else
        hotkey:SetText(GetBindingText(key, "KEY_", 1));
    end
end

function wsPopBarButton_Update()
	-- Special case code for bonus bar buttons
	-- Prevents the button from updating if the bonusbar is still in an animation transition
	if ( this.isBonus and this.inTransition ) then
		wsPopBarButton_UpdateUsable();
		wsPopBarButton_UpdateCooldown();
		return;
	end

	local action = wsPopBarButton_GetPagedID(this);
	local icon = getglobal(this:GetName().."Icon");
	local buttonCooldown = getglobal(this:GetName().."Cooldown");
	local texture = GetActionTexture(action);
	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		this.rangeTimer = -1;
		this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
		-- Save texture if the button is a bonus button, will be needed later
		if ( this.isBonus ) then
			this.texture = texture;
		end
	else
		icon:Hide();
		buttonCooldown:Hide();
		this.rangeTimer = nil;
		this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
		getglobal(this:GetName().."HotKey"):SetVertexColor(0.6, 0.6, 0.6);
	end
	wsPopBarButton_UpdateCount();
	if ( HasAction(action) ) then
		this:RegisterEvent("ACTIONBAR_UPDATE_STATE");
		this:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
		this:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
		this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
		this:RegisterEvent("PLAYER_AURAS_CHANGED");
		this:RegisterEvent("PLAYER_TARGET_CHANGED");
		this:RegisterEvent("UNIT_INVENTORY_CHANGED");
		this:RegisterEvent("CRAFT_SHOW");
		this:RegisterEvent("CRAFT_CLOSE");
		this:RegisterEvent("TRADE_SKILL_SHOW");
		this:RegisterEvent("TRADE_SKILL_CLOSE");
		this:RegisterEvent("PLAYER_ENTER_COMBAT");
		this:RegisterEvent("PLAYER_LEAVE_COMBAT");
		this:RegisterEvent("START_AUTOREPEAT_SPELL");
		this:RegisterEvent("STOP_AUTOREPEAT_SPELL");

		--Note:2008-4-9 Interface\AddOns\PopBar\PopBarFrame.lua:102: in function `wsPopBarButton_Update'
		if ( not this:GetAttribute("statehidden") and not this.wshide and InCombatLockdown()) then
			this:Show();
		end
		wsPopBarButton_UpdateState();
		wsPopBarButton_UpdateUsable();
		wsPopBarButton_UpdateCooldown();
		wsPopBarButton_UpdateFlash();
	else
		this:UnregisterEvent("ACTIONBAR_UPDATE_STATE");
		this:UnregisterEvent("ACTIONBAR_UPDATE_USABLE");
		this:UnregisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
		this:UnregisterEvent("UPDATE_INVENTORY_ALERTS");
		this:UnregisterEvent("PLAYER_AURAS_CHANGED");
		this:UnregisterEvent("PLAYER_TARGET_CHANGED");
		this:UnregisterEvent("UNIT_INVENTORY_CHANGED");
		this:UnregisterEvent("CRAFT_SHOW");
		this:UnregisterEvent("CRAFT_CLOSE");
		this:UnregisterEvent("TRADE_SKILL_SHOW");
		this:UnregisterEvent("TRADE_SKILL_CLOSE");
		this:UnregisterEvent("PLAYER_ENTER_COMBAT");
		this:UnregisterEvent("PLAYER_LEAVE_COMBAT");
		this:UnregisterEvent("START_AUTOREPEAT_SPELL");
		this:UnregisterEvent("STOP_AUTOREPEAT_SPELL");

		if ( this.showgrid == 0 ) then
			this:Hide();
		else
			buttonCooldown:Hide();
		end
	end

	-- Add a green border if button is an equipped item
	local border = getglobal(this:GetName().."Border");
	if ( IsEquippedAction(action) ) then
		border:SetVertexColor(0, 1.0, 0, 0.35);
		border:Show();
	else
		border:Hide();
	end

	if ( GameTooltip:IsOwned(this) ) then
		wsPopBarButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end

	-- Update Macro Text
	local macroName = getglobal(this:GetName().."Name");
	if ( not IsConsumableAction(action) and not IsStackableAction(action) ) then
		macroName:SetText(GetActionText(action));
	else
		macroName:SetText("");
	end

	-- Save the action so we don't update in OnUpdate()
	this.action = action;

	if(not InCombatLockdown()) then
	this:SetID(action) 
	end
end

function wsPopBarButton_ShowGrid(button)
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid+1;
	getglobal(button:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);

	if ( not button:GetAttribute("statehidden") and not this.wshide) then
		button:Show();
	end
end

function wsPopBarButton_HideGrid(button)	
	if ( not button ) then
		button = this;
	end
	button.showgrid = button.showgrid-1;
	if ( button.showgrid == 0 and not HasAction(wsPopBarButton_GetPagedID(button)) ) then
		button:Hide();
	end
end

function wsPopBarButton_UpdateState(button)
	if ( not button ) then
		button = this;
	end
	local actionID = wsPopBarButton_GetPagedID(button);
	if ( IsCurrentAction(actionID) or IsAutoRepeatAction(actionID) ) then
		button:SetChecked(1);
	else
		button:SetChecked(0);
	end
end

function wsPopBarButton_UpdateUsable()
	local icon = getglobal(this:GetName().."Icon");
	local normalTexture = getglobal(this:GetName().."NormalTexture");
	local isUsable, notEnoughMana = IsUsableAction(wsPopBarButton_GetPagedID(this));
	if ( isUsable ) then
		icon:SetVertexColor(1.0, 1.0, 1.0);
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
	elseif ( notEnoughMana ) then
		icon:SetVertexColor(0.5, 0.5, 1.0);
		normalTexture:SetVertexColor(0.5, 0.5, 1.0);
	else
		icon:SetVertexColor(0.4, 0.4, 0.4);
		normalTexture:SetVertexColor(1.0, 1.0, 1.0);
	end

	if ( IsActionInRange( wsPopBarButton_GetPagedID(this)) == 0 ) then
		icon:SetVertexColor(1.0, 0.1, 0.1);
	end
end

function wsPopBarButton_UpdateCount()
	local text = getglobal(this:GetName().."Count");
	local action = wsPopBarButton_GetPagedID(this);
	if ( IsConsumableAction(action) or IsStackableAction(action) ) then
		text:SetText(GetActionCount(action));
	else
		text:SetText("");
	end
end

function wsPopBarButton_UpdateCooldown()
	local cooldown = getglobal(this:GetName().."Cooldown");
	local start, duration, enable = GetActionCooldown(wsPopBarButton_GetPagedID(this));
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
end

function wsPopBarButton_OnEvent(event)
	if ( event == "ACTIONBAR_SLOT_CHANGED" ) then
		if ( arg1 == 0 or arg1 == wsPopBarButton_GetPagedID(this) ) then
			wsPopBarButton_Update();
		end
		return;
	end
	if ( event == "PLAYER_ENTERING_WORLD" or event == "ACTIONBAR_PAGE_CHANGED" ) then
		wsPopBarButton_Update();
		return;
	end
	if ( event == "UPDATE_BONUS_ACTIONBAR" ) then
		if ( this.isBonus ) then
			wsPopBarButton_Update();
		end
		return;
	end
	if ( event == "ACTIONBAR_SHOWGRID" ) then
		wsPopBarButton_ShowGrid();
		return;
	end
	if ( event == "ACTIONBAR_HIDEGRID" ) then
		wsPopBarButton_HideGrid();
		return;
	end
	if ( event == "UPDATE_BINDINGS" ) then
		wsPopBarButton_UpdateHotkeys(this.buttonType);
		return;
	end

	-- All event handlers below this line are only set when the button has an action

	if ( event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_AURAS_CHANGED" ) then
		wsPopBarButton_UpdateUsable();
		wsPopBarButton_UpdateHotkeys(this.buttonType);
	elseif ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == "player" ) then
			wsPopBarButton_Update();--fix
		end
	elseif ( event == "ACTIONBAR_UPDATE_STATE" ) then
		wsPopBarButton_UpdateState();
	elseif ( event == "ACTIONBAR_UPDATE_USABLE" or event == "UPDATE_INVENTORY_ALERTS" or event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		wsPopBarButton_UpdateUsable();
		wsPopBarButton_UpdateCooldown();
	elseif ( event == "CRAFT_SHOW" or event == "CRAFT_CLOSE" or event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" ) then
		wsPopBarButton_UpdateState();
	elseif ( event == "PLAYER_ENTER_COMBAT" ) then
		if ( IsAttackAction(wsPopBarButton_GetPagedID(this)) ) then
			wsPopBarButton_StartFlash();
		end
	elseif ( event == "PLAYER_LEAVE_COMBAT" ) then
		if ( IsAttackAction(wsPopBarButton_GetPagedID(this)) ) then
			wsPopBarButton_StopFlash();
		end
	elseif ( event == "START_AUTOREPEAT_SPELL" ) then
		if ( IsAutoRepeatAction(wsPopBarButton_GetPagedID(this)) ) then
			wsPopBarButton_StartFlash();
		end
	elseif ( event == "STOP_AUTOREPEAT_SPELL" ) then
		if ( wsPopBarButton_IsFlashing() and not IsAttackAction(wsPopBarButton_GetPagedID(this)) ) then
			wsPopBarButton_StopFlash();
		end
	end
end

function wsPopBarButton_SetTooltip()
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, this);
	else
		if ( this:GetParent() == MultiBarBottomRight or this:GetParent() == MultiBarRight or this:GetParent() == MultiBarLeft ) then
			GameTooltip:SetOwner(this, "ANCHOR_LEFT");
		else
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		end
		
	end
	
	if ( GameTooltip:SetAction(wsPopBarButton_GetPagedID(this)) ) then
		this.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		this.updateTooltip = nil;
	end
end

function wsPopBarButton_OnUpdate(elapsed)

	local hotkey = getglobal(this:GetName().."HotKey");
	local action = wsPopBarButton_GetPagedID(this);

	if ( this.action ~= action ) then
		wsPopBarButton_Update();
	end

	if ( hotkey:GetText() == RANGE_INDICATOR ) then
		if ( IsActionInRange(action) == 1 ) then
			hotkey:Hide();
		else
			hotkey:Show();
		end
	end

	if ( wsPopBarButton_IsFlashing() ) then
		this.flashtime = this.flashtime - elapsed;
		if ( this.flashtime <= 0 ) then
			local overtime = -this.flashtime;
			if ( overtime >= ATTACK_BUTTON_FLASH_TIME ) then
				overtime = 0;
			end
			this.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime;

			local flashTexture = getglobal(this:GetName().."Flash");
			if ( flashTexture:IsShown() ) then
				flashTexture:Hide();
			else
				flashTexture:Show();
			end
		end
	end
	
	-- Handle range indicator
	if ( this.rangeTimer ) then
		this.rangeTimer = this.rangeTimer - elapsed;

		if ( this.rangeTimer <= 0 ) then
			wsPopBarButton_UpdateUsable();
			local count = getglobal(this:GetName().."HotKey");
			if ( IsActionInRange(action) == 0 ) then
				count:SetVertexColor(1.0, 0.1, 0.1);
			else
				count:SetVertexColor(0.6, 0.6, 0.6);
			end
			this.rangeTimer = TOOLTIP_UPDATE_TIME;
		end
	end

	if ( not this.updateTooltip ) then
		return;
	end

	this.updateTooltip = this.updateTooltip - elapsed;
	if ( this.updateTooltip > 0 ) then
		return;
	end

	if ( GameTooltip:IsOwned(this) ) then
		wsPopBarButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end
end

function wsPopBarButton_GetPagedID(button)
	if(not button) then
		button = this;
	end

	local name = button:GetName();
	local _,_,num = string.find(name,"wsPopBarButton(.+)")
	local num = tonumber(num)

	local butId = num;
	local spage = 0;
	if (wsPopBar) then
		spage = tonumber(wsPopBar.StartPage) - 1;
	end
	local id = math.fmod(butId, 100);
	local barid = math.floor(butId/100);
	local resId = math.fmod((barid-1+spage), 10)*12 + id;
	if(InCombatLockdown()) then
		resId = resId + (GetActionBarPage() - 1) * NUM_ACTIONBAR_BUTTONS;
		if(resId > 120) then
			resId = resId -120;
		end
	end
	return resId;
end

function wsPopBarButton_UpdateFlash()
	local action = wsPopBarButton_GetPagedID(this);
	if ( (IsAttackAction(action) and IsCurrentAction(action)) or IsAutoRepeatAction(action) ) then
		wsPopBarButton_StartFlash();
	else
		wsPopBarButton_StopFlash();
	end
end

function wsPopBarButton_StartFlash()
	this.flashing = 1;
	this.flashtime = 0;
	wsPopBarButton_UpdateState();
end

function wsPopBarButton_StopFlash()
	this.flashing = 0;
	getglobal(this:GetName().."Flash"):Hide();
	wsPopBarButton_UpdateState();
end

function wsPopBarButton_IsFlashing()
	if ( this.flashing == 1 ) then
		return 1;
	else
		return nil;
	end
end

function wsPopBarFrame_Update()
	if (wsPopBar) then
		for row=1, 7 do
			for col=1, 12 do
				local btnName = wsPopBarButton_GetBindButton(row, col);
				if (btnName) then
					local button = getglobal(btnName);
					if (button) then
						button:Hide();
						if(row > wsPopBar.Rows) then
							button.wshide = 1;
						else
							button.wshide = nil;
						end

						if(wsPopBar.HideEmpty == "1") then
							button.showgrid = 0;
						else
							button.showgrid = 1;
						end
						
						local action = wsPopBarButton_GetPagedID(button);
						if(not button.wshide and (button.showgrid == 1 or (button.showgrid == 0 and HasAction(action)))) then
							button:Show();
						end
					end
				end
			end
		end
	end
end

function wsPopBarFrame_Toggle(toggle)
	if (toggle) then
		wsPopBarFrame:Show();
--		STATE_MultiBar2 = nil;
		MultiBarBottomRight:Hide()
	else
		wsPopBarFrame:Hide();
	end
	wsPopBarFrame_Update();
end

function wsPopBarFrame_OnUpdate(elapsed)

end