--	WoWShell Utilities
--		A set of useful functions
--
--	by Robin
--
--	Revision: 2
--	Last Author: Robin
--	Last Change: 2004-12-22 11:30
--
--	Changelog:
--		Rev 2:
--			Added IsTable();


--WOWSHELL_DEBUG = 1;

-- Output functions
function Print(msg, r, g, b, frame, id)
	if (not r) then r = 1.0; end
	if (not g) then g = 1.0; end
	if (not b) then b = 1.0; end
	if (not frame) then frame = DEFAULT_CHAT_FRAME; end
	if (frame) then
		if (not id) then
			frame:AddMessage(msg,r,g,b);
		else
			frame:AddMessage(msg,r,g,b,id);
		end
	end
end

-- Prints a table in an organized format
function PrintTable(table, rowname, level)
	if ( rowname == nil ) then rowname = "ROOT"; end
	if ( level == nil ) then level = 1; end

	local msg = "";
	for i=1,level, 1 do
		msg = msg .. "   ";
	end

	if ( table == nil ) then Print (msg.."["..rowname.."] := nil "); return end
	if ( type(table) == "table" ) then
		Print (msg..rowname.." { ");
		for k,v in table do
			PrintTable(v,k,level+1);
		end
		Print(msg.."} ");
	elseif (type(table) == "function" ) then
		Print(msg.."["..rowname.."] => {{FunctionPtr*}}");
	elseif (type(table) == "userdata" ) then
		Print(msg.."["..rowname.."] => {{UserData}}");
	elseif (type(table) == "boolean" ) then
		local value = "true";
		if ( not table ) then
			value = "false";
		end
		Print(msg.."["..rowname.."] => "..value);
	else
		Print(msg.."["..rowname.."] => "..table);
	end
end

-- DebugPrint takes a message and a debug key.
--
-- The default is "WOWSHELL_DEBUG", but you can set it to any
-- value you like. The result is that if setglobal(debugkey,1);
-- has been called, your debug message will be output.
--
function DebugPrint(msg, debugkey)
	if ( not debugkey ) then debugkey = "WOWSHELL_DEBUG"; end

	if ( getglobal(debugkey) == 1 ) then
		local msg_debug=type(msg);
		if ( type(msg) == "string" or type(msg) == "number" ) then
			msg_debug = msg;
		elseif ( type(msg) == "boolean" ) then
			if (msg) then
				msg_debug = "true";
			else
				msg_debug = "false";
			end
		end
		local hour, minute=GetGameTime();
		Print(string.format("[%02d:%02d]DBG: %s", hour, minute, msg_debug), 1, 1, 0.5, DEFAULT_CHAT_FRAME);
	end
end

-- Prints an error message
function ErrorPrint(msg)
	local info = ChatTypeInfo["YELL"];
	UIErrorsFrame:AddMessage(msg, info.r, info.g, info.b, 1.0, UIERRORS_HOLD_TIME);
end

--Returns true if variable is not nill and a table
function IsTable(var)
	if (var and (type(var) == "table")) then
		return true;
	end
	return false;
end