--[[
  ****************************************************************
  Scrolling Combat Text
  Event Config File
  ****************************************************************

  PLEASE NOTE:
    Custom events are radically different than from previous versions.
    Please see examples below to learn how to most everything in the
    new system

  Description:
    This file is used to setup custom events for use in SCT. Due to the
    combat log changes in 2.4, custom events are very differnt. You now
    must specificy what type of event for every event, along with various
    optional settings to get at what you want. Please read all the paramaters
    below in detail and review the examples.

     display:   This is basically the text you want SCT to display
                whenever the event occurs. for captured data you
                use *n where n is the index of the captured data you
                want to display. See the table below for all available
                arguments.

      search:   in most cases now, this is the spell/skill that you are
                looking for. However this is optional and not always needed
                depending on what you are wanting to. Use ACTION_SWING for
                regular melee

        type:   Indicates the type of event. The Current
                allowed types are:

                "BUFFSELF" - When you gain a buff or debuff
                "BUFFTARGET" - When your target gains a buff or debuff
                "BUFFPET" - When your pet gains a buff or debuff
                "FADESELF" - When you lose a buff or debuff
                "FADETARGET" - When your target loses a buff or debuff
                "FADEPET" - When your pet loses a buff or debuff
                "MISSSELF" - When something misses you
                "MISSTARGET" - When you or your pet miss your target
                "MISSPET" - When something misses your pet
                "HEALSELF" - When you perform healing or are healed
                "HEALTARGET" - When your target is healed
                "HEALPET" - When your pet is healed
                "DAMAGESELF" - When you are damaged
                "DAMAGETARGET" - When you or your pet damage your target
                "DAMAGEPET" - When your pet is damaged

                                                            CAPTURES
                TYPE            OPTIONS                     arg1       arg2        arg3        arg4
                "BUFFSELF"      buffcount                   spellname  sourcename  nil         count
                "BUFFTARGET"    buffcount, selfonly         spellname  nil         targetname  count
                "BUFFPET"       buffcount                   spellname  nil         targetname  count
                "FADESELF"      buffcount                   spellname  sourcename  nil         count
                "FADETARGET"    buffcount                   spellname  nil         targetname  count
                "FADEPET"       buffcount                   spellname  nil         targetname  count
                "MISSSELF"      misstype                    spellname  sourcename  targetname  misstype
                "MISSTARGET"    misstype, selfonly, petonly spellname  sourcename  targetname  misstype
                "MISSPET"       misstype                    spellname  sourcename  targetname  misstype
                "HEALSELF"      critical                    spellname  sourcename  targetname  amount
                "HEALTARGET"    critical                    spellname  sourcename  targetname  amount
                "HEALPET"       critical                    spellname  sourcename  targetname  amount
                "DAMAGESELF"    critical                    spellname  sourcename  targetname  amount
                "DAMAGETARGET"  critical, selfonly, petonly spellname  sourcename  targetname  amount
                "DAMAGEPET"     critical                    spellname  sourcename  targetname  amount

                OPTIONS DESCRIPTIONS
                buffcount: if used, this limits the event to only fire when the count of the
                           buff is this value. Usefull for debuffs like Sunder Armor, Deadly Poision, etc...

                selfonly:  if set to 1 or true, this will make the event only trigger if you caused it.

                petonly:   if set to 1 or true, this will make the event only trigger if your pet caused it

                misstype:  the type of the miss. DODGE, ABSORB, RESIST, PARRY, MISS, BLOCK,
                           REFLECT, DEFLECT, IMMUNE, EVADE

                critical:  if set to 1 or true, the event will only trigger if the event was a crit


      r, g, b:  These are the color settings used to select the
                color you want the event to display in. r = red,
                g = green, b = blue. Some common colors:

                Red:        r=256/256,  g=0/256,    b=0/256
                Green:      r=0/256,    g=256/256,  b=0/256
                Blue:       r=0/256,    g=0/256,    b=256/256
                Yellow:     r=256/256,  g=256/256,  b=0/256
                Magenta:    r=256/256,  g=0/256,    b=256/256
                Cyan:       r=0/256,    g=256/256,  b=256/256

      iscrit:   this will make the event appear as a crit event,
                so it will be sticky or large font. You only need
                to set this if you want it used. iscrit=1

        icon:   this will attempt to show the spell icon for the event
                if it is appropriate. icon=1

    override:   if set to false, the event will be displayed but any normal
                event will also still trigger. Defaults to true.

       frame:   Determines what frame to show the event in. Default is Frame1.
                The only values allowed are:

                SCT.FRAME1
                SCT.FRAME2
                SCT.FRAME3 (SCTD)
                SCT.MSG (SCT message frame)

       class:   This allows you to filter events by class. Put only
                the classes you want to see the even here. They must
                be in LUA table format.

                Examples:

                Warrior only:         class={"Warrior"}
                Warrior and Shaman:   class={"Warrior", "Shaman"}

     anitype:   this allows you to specify the type of animation to
                use for an event.

                1 - Vertical
                2 - Rainbow
                3 - Horizontal
                4 - Angled Down
                5 - Angled Up
                6 - Sprinkler
                7 - HUD Curved
                8 - HUd Angled

        sound:  (advanced users only)
                plays a sound built into WoW. Please see here for a
                list of values: http://www.wowwiki.com/API_PlaySound

                Examples:
                sound="TellMessage"
                sound="GnomeExploration"

    soundwave:  (advanced users only)
                plays a wave file in the path you choose. You can use
                in game ones if you know the path, or custom ones you
                put it in an addon folder.

                Examples:
                In game:      soundwave="Sound\\Spells\\ShaysBell.wav"
                Custom:       soundwave="Interface\\AddOns\\MyAddOn\\mysound.wav"

  Note:
    Make sure you increase the key count ([1], [2], etc...) for
    each new event you add. Feel free to delete any of the already
    provided events if you know you will never need them or you
    dont want them to display. You may also place -- in front
    of any of them to comment them out.

  ****************************************************************]]

local sct_Event_Config = {

-- To remove an event, simply add -- in front of the line.

[1] = {display="Clearcast!", type="BUFFSELF", search="Clearcast", icon=1, r=256/256, g=256/256, b=0/256, iscrit=1, class={"Mage","Shaman","Druid","Priest"}},
[2] = {display="Flurry!", type="BUFFSELF", search="Flurry", r=128/256, g=0/256, b=0/256, class={"Warrior","Shaman"}},
[3] = {display="Lightning Shield!", type="BUFFSELF", search="Lightning Shield", r=0/256, g=0/256, b=256/256,class={"Shaman"}},
[4] = {display="Nightfall!", type="BUFFSELF", search="Shadow Trance", r=0/256, g=128/256, b=128/256, class={"Warlock"}},
[5] = {display="Overpower!", type="MISSTARGET", override=false, misstype="DODGE", r=256/256, g=256/256, b=0/256, iscrit=1, selfonly=1, class={"Warrior"}},
[6] = {display="Enraged!", type="BUFFSELF", search="Enrage", r=128/256, g=256/256, b=128/256, iscrit=1, class={"Warrior"}},
[7] = {display="Crusader!", type="BUFFSELF", search="Holy Strength", r=128/256, g=128/256, b=256/256, iscrit=1,  class={"Warrior","Rogue","Paladin","Shaman","Hunter"}},
[8] = {display="Spirit Tap!", type="BUFFSELF", search="Spirit Tap", r=128/256, g=128/256, b=150/256, iscrit=1,  class={"Priest"}},
[9] = {display="Soul Shard!", search="You(.+)Soul Shard", r=0/256, g=128/256, b=128/256, iscrit=1, class={"Warlock"}},
[10] = {display="Mace Stun!", type="BUFFTARGET", search="Mace Stun",r=256/256, g=256/256, b=0/256, iscrit=1, selfonly=1, class={"Rogue","Warrior"}},
[11] = {display="Rooted!", type="BUFFTARGET", search="Improved Hamstring", r=0/256, g=128/256, b=0/256, iscrit=1, selfonly=1, class={"Warrior"}},
[12] = {display="Full Sunder!", type="BUFFTARGET", search="Sunder Armor", buffcount=5, r=0/256, g=256/256, b=0/256, iscrit=1, class={"Warrior"}},
[13] = {display="Envenom!", type="BUFFTARGET", search="Deadly", buffcount=5, r=256/256, g=0/256, b=256/256, iscrit=1, selfonly=1, class={"Rogue"}},
[14] = {display="Frostbite!", type="BUFFTARGET", search="Frostbite", icon=1, r=75/256, g=150/256, b=225/256, iscrit=1, selfonly=1, class={"Mage"}},
[15] = {display="", type="MISSTARGET", search="Deep Wound", selfonly=1, r=0/256, g=0/256, b=0/256, class={"Warrior"}},
[16] = {display="Blackout!", type="BUFFTARGET", search="Blackout", r=128/256, g=128/256, b=150/256, iscrit=1, selfonly=1, class={"Priest"}},
[17] = {display="Impact!", type="BUFFTARGET", search="Impact", r=128/256, g=64/256, b=64/256, iscrit=1, selfonly=1, class={"Mage"}},
[18] = {display="Feint Failed!", type="MISSTARGET", search="Feint", r=255/256, g=153/256, b=51/256, iscrit=1, selfonly=1, class={"Rogue"}},

--Captured data examples. Remove, change, or comment out the ones you don't want.

--Spam Stoppers
--[X] = {display="", type="HEALSELF", search="Bloodthirst", r=256/256, g=256/256, b=0/256, class={"Warrior"}},
--[X] = {display="", type="HEALSELF", search="Vampiric Embrace", r=256/256, g=256/256, b=0/256, class={"Priest"}},

--Other common requests
--[19] = {display="*1 - (*4)", type="DAMAGETARGET", search="Arcane Shot", iscrit=1, selfonly=1, r=256/256, g=256/256, b=0/256, class={"Hunter"}},
--[19] = {display="Critical! (*1: *4)", type="DAMAGETARGET", critical=1, iscrit=1, selfonly=1, r=256/256, g=256/256, b=0/256, class={"Rogue","Hunter"}},

--NOTE: If you are wanting SCT to show your damage, I suggest you grab SCT - Damage instead. You can
--get it here: http://www.wowinterface.com/downloads/fileinfo.php?id=4913

};


--[[
  ****************************************************************
  This list of Wow Events are used for searching custom events.
  They are only events that are not normally used by SCT.
  By default they are commented out. To enable an event to be
  searched by SCT, remove the "--" before the event, save, and
  then reload/relog. if for some reason you do not see the event
  you need here, you can find a full list of them here:
  http://www.wowwiki.com/Events_%28API%29

  To figure out what event a chat message is, set SCT_Event_Debug
  to true. You will now get a chat message saying what event is
  being fired for every event. Please Note this may cause "tainting"
  but since you should only use it to find the event you need, once
  you turn it back off everything will be fine.
  ****************************************************************]]

local sct_Event_Debug = false;

local sct_Event_List = {

--experience
--"CHAT_MSG_COMBAT_XP_GAIN",

--trade skills
--"CHAT_MSG_SPELL_TRADESKILLS",

--monster emotes
--"CHAT_MSG_MONSTER_EMOTE",
--"CHAT_MSG_MONSTER_SAY",
--"CHAT_MSG_MONSTER_WHISPER",
--"CHAT_MSG_MONSTER_YELL",

--chat channels
--"CHAT_MSG_DND",
--"CHAT_MSG_EMOTE",
--"CHAT_MSG_GUILD",
--"CHAT_MSG_IGNORED",
--"CHAT_MSG_LOOT",
--"CHAT_MSG_OFFICER",
--"CHAT_MSG_PARTY",
--"CHAT_MSG_RAID",
--"CHAT_MSG_SAY",
--"CHAT_MSG_SYSTEM",
--"CHAT_MSG_TEXT_EMOTE",
--"CHAT_MSG_WHISPER",
--"CHAT_MSG_WHISPER_INFORM",
--"CHAT_MSG_YELL",

--battlegrounds
--"CHAT_MSG_BG_SYSTEM_ALLIANCE",
--"CHAT_MSG_BG_SYSTEM_HORDE",
--"CHAT_MSG_BG_SYSTEM_NEUTRAL",

--misc
--"CHAT_MSG_COMBAT_ERROR",
--"CHAT_MSG_COMBAT_MISC_INFO",
--"CHAT_MSG_SPELL_FAILED_LOCALPLAYER",
}

--Don't mess with these =)
SCT.EventConfig = sct_Event_Config;
SCT.EventDebug = sct_Event_Debug;
SCT.EventList = sct_Event_List;