local SCT = SCT

local CUSTOM_EVENTS = {
  "BUFFSELF",
  "BUFFTARGET",
  "BUFFPET",
  "FADESELF",
  "FADETARGET",
  "FADEPET",
  "MISSSELF",
  "MISSTARGET",
  "MISSPET",
  "HEALSELF",
  "HEALTARGET",
  "HEALPET",
  "DAMAGESELF",
  "DAMAGETARGET",
  "DAMAGEPET",
  "CHAT",
}

--LUA calls
local pairs = pairs
local ipairs = ipairs
local strfind = strfind
local string_gsub = string.gsub
local string_gmatch = string.gmatch
local tremove = tremove
local table_sort = table.sort
local strupper = strupper
local tinsert = function(t,v)
  t[#t+1] = v
end

local GetSpellInfo = GetSpellInfo
local UnitDebuff = UnitDebuff

------------------------------
---Sort a table by its keys.
---stolen from http://www.lua.org/pil/19.3.html
function SCT:PairsByKeys(t, f)
  local a = {}
  for n in pairs(t) do tinsert(a, n) end
  table_sort(a, f)
  local i = 0      -- iterator variable
  local iter = function ()   -- iterator function
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end
  return iter
end

-------------------------
--Cache Custom Events
function SCT:CacheCustomEvents()
  local classfound, key, key2, class, value

  --cache it index order
  self.CustomEvents = {}
  self.CustomCombatEvents = {}
  for k,v in pairs(CUSTOM_EVENTS) do
    self.CustomCombatEvents[v] = {}
  end

  for key, value in self:PairsByKeys(self.EventConfig) do
    --set override
    if value.override ~= false then
      value.override = true
    end
    --default class found to true
    classfound = true
    --check if they want to see it for this class
    if (value.class) then
      --if want to filter by class, default to false
      classfound = false
      for key2, class in pairs(value.class) do
        if (strlower(UnitClass("player")) == strlower(class)) then
          classfound = true
          break
        end
      end
    end
    if (classfound) then
      --seperate events based on type
      if self.CustomCombatEvents[value.type] then
        tinsert(self.CustomCombatEvents[value.type], value)
      else
        tinsert(self.CustomEvents, value)
      end
    end
  end
  -- assign default ranking. rank is used to keep them in proper parsing order
  for key, value in ipairs(self.CustomEvents) do
    value.rank = key
  end
end

----------------------
--Find a custom capture event message
function SCT:CustomEventSearch(larg1)
  if (not self.db.profile["CUSTOMEVENTS"]) then
    return false
  end
  local key, value
  local carg1, carg2, carg3, carg4, carg5
  --First check for found events
  if (self.FoundCustomEvents[event]) then
    for key, value in ipairs(self.FoundCustomEvents[event]) do
      for carg1, carg2, carg3, carg4, carg5 in string_gmatch(larg1, value.search) do
        self:FormatCustomEvent(value, nil, carg1, carg2, carg3, carg4, carg5)
        return value.override
      end
    end
  end
  --Now check for unfound events
  for key, value in ipairs(self.CustomEvents) do
    for carg1, carg2, carg3, carg4, carg5 in string_gmatch(larg1, value.search) do
      -- flag it as found by adding to found table by event, then sort table by rank
      if (self.FoundCustomEvents[event]) then
        tinsert(self.FoundCustomEvents[event], value)
        table_sort(self.FoundCustomEvents[event], function(a,b) return a.rank<b.rank end)
      else
        local eventArr = {}
        tinsert(eventArr, value)
        self.FoundCustomEvents[event] = eventArr
      end
      self:FormatCustomEvent(value, nil, carg1, carg2, carg3, carg4, carg5)
      --remove it from table
      tremove(self.CustomEvents, key)
      return value.override
    end
  end
  return false
end

----------------------
--Find a custom combat event
function SCT:CustomCombatEventSearch(event, toPlayer, fromPlayer, toPet, fromPet, toTarget, sourceName, destName, ...)
  --locals
  local prefix, suffix, texture
  local amount, critical, spellId, spellName, spellSchool, missType, auraType, source

  --set suffix based on flags
  if toTarget then
    suffix = "TARGET"
  elseif toPlayer or fromPlayer then
    suffix = "SELF"
  elseif toPet or fromPet then
    suffix = "PET"
  else
    return false
  end

  ---------buffs------------
  if strfind(event, "_AURA") then
    if strfind(event, "_APPLIED") then
      prefix = "BUFF"
    elseif strfind(event, "_REMOVED") then
      prefix = "FADE"
    else
      return false
    end
    spellId, spellName, spellSchool, auraType, amount = select(1, ...)
    texture = select(3, GetSpellInfo(spellId))
    return self:AuraCustomEvent(self.CustomCombatEvents[prefix..suffix], spellName, amount, auraType, sourceName, destName, texture)
  ---------miss-------------
  elseif strfind(event, "_MISSED") then
    prefix = "MISS"
    if event == "SWING_MISSED" then
      missType = select(1, ...)
      spellName = ACTION_SWING
    else
      spellId, spellName, spellSchool, missType = select(1, ...)
      texture = select(3, GetSpellInfo(spellId))
    end
    if toTarget then
      source = fromPlayer and "PLAYER" or fromPet and "PET"
    end
    return self:MissCustomEvent(self.CustomCombatEvents[prefix..suffix], spellName, missType, sourceName, destName, source, texture)
  ---------heals-------------
  elseif strfind(event, "_HEAL") then
    prefix = "HEAL"
    spellId, spellName, spellSchool, amount,	critical = select(1, ...)
    texture = select(3, GetSpellInfo(spellId))
    return self:HealDamageCustomEvent(self.CustomCombatEvents[prefix..suffix], spellName, critical, amount, sourceName, destName, "PLAYER", texture)
  ---------damage-------------
  elseif strfind(event, "_DAMAGE") then
    prefix = "DAMAGE"
    if event == "SWING_DAMAGE" then
      amount, _, _, _, _, critical = select(1, ...)
      spellName = ACTION_SWING
    else
      spellId, spellName, spellSchool, amount, _, _, _, _, critical = select(1, ...)
      texture = select(3, GetSpellInfo(spellId))
    end
    if toTarget then
      source = fromPlayer and "PLAYER" or fromPet and "PET"
    end
    return self:HealDamageCustomEvent(self.CustomCombatEvents[prefix..suffix], spellName, critical, amount, sourceName, destName, source, texture)
  end
end

----------------------
--Find a aura custom event
function SCT:AuraCustomEvent(tab, spellName, count, auraType, sourceName, destName, icon)
  local found
  --loop through each buff
  for key, value in ipairs(tab) do
    found = true
    --if they care about skill and skill is not found
    if value.search and spellName and not strfind(spellName, value.search) then
      found = false
    end
    --if they want self only debuffs then
    if found and auraType == "DEBUFF" and value.selfonly and not self:CheckSelfDebuff(spellName, "target") then
      found = false
    end
    --if they care about miss type and miss type is not found
    if found and value.buffcount and value.buffcount ~= count then
      found = false
    end
    --if still ok, then send custom event
    if found then
      self:FormatCustomEvent(value, icon, spellName, sourceName, destName, count)
      return value.override
    end
  end
  return false
end

----------------------
--Find a aura custom event
function SCT:MissCustomEvent(tab, spellName, missType, sourceName, destName, source, icon)
  local found
  --loop through each miss
  for key, value in ipairs(tab) do
    found = true
    --if they care about skill and skill is not found
    if value.search and spellName and not strfind(spellName, value.search) then
      found = false
    end
    --if they care about miss type and miss type is not found
    if found and value.misstype and strupper(value.misstype) ~= missType then
      found = false
    end
    --if they care about source
    if found and value.selfonly and source ~= "PLAYER" then
       found = false
    end
    if found and value.petonly and source ~= "PET" then
       found = false
    end
    --if still ok, then send custom event
    if found then
      self:FormatCustomEvent(value, icon, spellName, sourceName, destName, missType)
      return value.override
    end
  end
  return false
end

----------------------
--Find a heal custom event
function SCT:HealDamageCustomEvent(tab, spellName, critical, amount, sourceName, destName, source, icon)
  local found
  --loop through each event
  for key, value in ipairs(tab) do
    found = true
    --if they care about skill and skill is not found
    if value.search and spellName and not strfind(spellName, value.search) then
      found = false
    end
    --if they care about crits and crit is not found
    if found and value.critical and not critical then
      found = false
    end
    --if they care about source
    if found and value.selfonly and source ~= "PLAYER" then
       found = false
    end
    if found and value.petonly and source ~= "PET" then
       found = false
    end
    --if still ok, then send custom event
    if found then
      self:FormatCustomEvent(value, icon, spellName, sourceName, destName, amount)
      return value.override
    end
  end
  return false
end

----------------------
--See if the debuff is ours
function SCT:CheckSelfDebuff(spellName, unit)
  local i = 1
  while true do
    local name, rank, icon, count, dispelType, duration, timeLeft = UnitDebuff(unit, i)
    if not name then break end
    if name == spellName and timeLeft then
      return true
    end
    i = i + 1
  end
  return false
end

----------------------
--Format and print a custom event
local currentcolor
function SCT:FormatCustomEvent(value, icon, carg1, carg2, carg3, carg4, carg5)
  local strTempMessage, replace, count = value.display, false, 0
  --if there are capture args
  if carg1 then
     strTempMessage, count = string_gsub(strTempMessage, "*1", carg1)
     if count > 0 then replace = true end
  end
  if carg2 then
     strTempMessage, count = string_gsub(strTempMessage, "*2", carg2)
     if count > 0 then replace = true end
  end
  if carg3 then
     strTempMessage, count = string_gsub(strTempMessage, "*3", carg3)
     if count > 0 then replace = true end
  end
  if carg4 then
     strTempMessage, count = string_gsub(strTempMessage, "*4", carg4)
     if count > 0 then replace = true end
  end
  if carg5 then
     strTempMessage, count = string_gsub(strTempMessage, "*5", carg5)
     if count > 0 then replace = true end
  end
  --check for dupes, filters out duplicate skills
  if not replace and self:CheckSkill(strTempMessage) then return end
  --get color
  currentcolor = {r = 1.0, g = 1.0, b = 1.0}
  if (value.r and value.g and value.b) then
    currentcolor.r,currentcolor.g,currentcolor.b = value.r,value.g,value.b
  end
  --show icon
  if not value.icon then icon = nil end
  --play sound
  if value.sound then PlaySound(value.sound) end
  --play soundwave
  if value.soundwave then PlaySoundFile(value.soundwave) end
  --display
  self:DisplayCustomEvent(strTempMessage, currentcolor, value.iscrit, value.frame or 1, value.anitype, icon)
end

----------------------
--Display the Text based on message flag for custom events
function SCT:DisplayCustomEvent(msg1, color, iscrit, frame, anitype, icon)
  if (frame == SCT.MSG) then
    self:DisplayMessage( msg1, color )
  else
    self:DisplayText(msg1 , color, iscrit, "event", frame or 1, anitype, nil, icon)
  end
end